<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

/**
 * @property Inbank $module
 */
class InbankCallbackModuleFrontController extends ModuleFrontController
{
    public function display()
    {
        $hmac = Tools::getValue('hmac');
        $timestamp = Tools::getValue('timestamp');
        $message = json_decode(Tools::getValue('message'));
        
        try {
            $order_id = InbankTools::getOrderIdByReference($message->purchase_reference);
        } catch (Exception $e) {
            InbankLogs::addLog(
                0,
                $this->module->l('There was a problem trying to find the correct order from Inbank callback.'),
                InbankLogs::TYPE_ERROR,
                '',
                'CallbackFrontController',
                $e->getMessage()
            );
            die("There was a problem trying to find the correct order from Inbank callback");
        }

        $order = Db::getInstance()->getRow(
            'SELECT module, reference FROM '._DB_PREFIX_.'orders WHERE id_order = '.(int)$order_id
        );
        if ($this->module->name !== $order['module']) {
            InbankLogs::addLog(
                $order_id,
                $this->module->l('Order verification failed'),
                InbankLogs::TYPE_ERROR,
                '',
                'CallbackFrontController'
            );
            die('Order verification failed');
        }

        $details = InbankDetail::getDetailsByOrderId($order_id);
        if (empty($details))
        {
            InbankLogs::addLog(
                $order_id,
                $this->module->l('There was a problem trying to find Inbank Order detail in database.'),
                InbankLogs::TYPE_ERROR,
                '',
                'CallbackFrontController'
            );
            die('There was a problem trying to find Inbank Order detail in database.');
        }
        $id_payment = $details->id_payment;

        $payment = new InbankPayment($id_payment);
        $current_key = $payment->debug_mode ? $payment->api_key_debug : $payment->api_key;
        if ($message->uuid !== $details->session_uuid ||
            !InbankTools::verifyHmac(
                $hmac,
                $timestamp,
                Tools::stripslashes(Tools::getValue('message')),
                $current_key
            )
        ) {
            InbankLogs::addLog(
                $order_id,
                $this->module->l(
                    'Failed to verify hmac or local session ID doesn\'t match the ID we received in callback.'
                ),
                InbankLogs::TYPE_ERROR,
                '',
                'CallbackFrontController'
            );
            die("Failed to verify hmac or local session id doesn't match the id we received");
        }
        $new_order_status = $message->status;

        //loading up inbank product
        $class_name = $payment->class_name;
        $inbank_product = new $class_name($payment, $this);

        //Checking payment info. some inbank products don't need this checking, therefore they won't have this method.
        if ($payment->needs_approval && method_exists($class_name, "checkPaymentStatus")) {
            try {
                $new_status = $inbank_product->checkPaymentStatus($details);
                if (isset($new_status->credit_contract_uuid)) {
                    $details->contract_uuid = $new_status->credit_contract_uuid;
                    $details->save();
                }
                $new_order_status = $new_status->status;
            } catch (Exception $e) {
                //don't need to end processing here, we can continue and try to get cotract uuid later.
                InbankLogs::addLog(
                    $order_id,
                    $this->module->l('Something went wrong trying to get contract ID. Check the order manually.'),
                    InbankLogs::TYPE_WARNING,
                    '',
                    'CallbackFrontController',
                    $e->getMessage()
                );
            }
        }

        $new_order_status_id = (int)Configuration::get("INBANK_".Tools::strtoupper($new_order_status));
        $cancelled_order_status_id = (int)Configuration::get("INBANK_CANCELLED");
        // second request is made to get newest data from DB as few concurrent requests may be performing the same action
        $current_state = (int)Db::getInstance()->getValue(
            'SELECT current_state FROM '._DB_PREFIX_.'orders WHERE id_order = '.(int)$order_id
        );
        //checking if status changed
        if ($new_order_status !== 'pending' && $current_state != $cancelled_order_status_id && $new_order_status_id !== $current_state) {
            try {
                $order_history = new OrderHistory();
                $order_history->id_order = $order_id;
                $order_history->changeIdOrderState($new_order_status_id, $order_id);
                $order_history->addWithemail(true, array(
                    'order_name' => $order['reference'],
                ));
            } catch (Exception $e) {
                InbankLogs::addLog(
                    $order_id,
                    $this->module->l('Failed to change order status.'),
                    InbankLogs::TYPE_WARNING,
                    '',
                    'CallbackFrontController',
                    $e->getMessage()
                );
                die("Failed to change order status.");
            }
        }
        $details->last_state = $new_order_status;
        $details->timestamp = $timestamp;
        $details->save();

        // adding payment
        if ($new_order_status == 'completed')
        {
            $order = new Order($order_id);
            if (empty($order->getOrderPayments()))
                $order->addOrderPayment($order->total_paid_tax_incl);
        }

        // adding log record
        InbankLogs::addLog(
            $order_id,
            $this->module->l('Got callback for order ' . $order_id . ' with state ' . $new_order_status),
            InbankLogs::TYPE_INFO,
            '',
            'callback'
        );

        die();
    }
}
