<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

/**
 * @property Inbank $module
 */
class InbankCancelModuleFrontController extends ModuleFrontController
{
    public function display()
    {
        $hmac = Tools::getValue('hmac');
        $timestamp = Tools::getValue('timestamp');
        $message = json_decode(Tools::getValue('message'));

        //trying to get order id from inbank redirect
        try {
            $order_id = InbankTools::getOrderIdByReference($message->purchase_reference);
        } catch (Exception $e) {
            InbankLogs::addLog(
                0,
                $this->module->l('There was a problem trying to find the correct order from Inbank cancel url.'),
                InbankLogs::TYPE_ERROR,
                '',
                'CancelFrontController',
                $e->getMessage()
            );
        }

        // checking if order exists
        $order = new Order($order_id);
        if ($this->module->name !== $order->module) {
            InbankLogs::addLog(
                $order_id,
                $this->module->l('Order verification failed'),
                InbankLogs::TYPE_ERROR,
                '',
                'ReturnFrontController'
            );
            Tools::redirect($this->context->link->getPageLink('index'));
        }

        // checking order detail in inbank_detail table
        $details = InbankDetail::getDetailsByOrderId($order_id);
        if (empty($details))
        {
            InbankLogs::addLog(
                $order_id,
                $this->module->l('There was a problem trying to find Inbank Order detail in database.'),
                InbankLogs::TYPE_ERROR,
                '',
                'CancelFrontController'
            );

            Tools::redirect(Tools::getHttpHost(true) . __PS_BASE_URI__);
        }

        // verifying hmac
        $id_payment = $details->id_payment;
        $payment = new InbankPayment($id_payment);
        $current_key = $payment->debug_mode ? $payment->api_key_debug : $payment->api_key;
        if ($message->uuid !== $details->session_uuid
            || !InbankTools::verifyHmac(
                $hmac,
                $timestamp,
                Tools::stripslashes(Tools::getValue('message')),
                $current_key
            )
        ) {
            InbankLogs::addLog(
                $order_id,
                $this->module->l(
                    'Failed to verify hmac or local session ID doesn\'t match the ID we received in cancel url.'
                ),
                InbankLogs::TYPE_ERROR,
                '',
                'CancelFrontController'
            );

            Tools::redirect(Tools::getHttpHost(true) . __PS_BASE_URI__);
        }

        $id_cancel_order_state = (int)Configuration::get("INBANK_CANCELLED");
        $order_state = Db::getInstance()->getRow(
            'SELECT current_state, reference FROM '._DB_PREFIX_.'orders WHERE id_order = '.(int)$order_id
        );

        if ($id_cancel_order_state != (int)$order_state['current_state']) {
            try {
                $order_history = new OrderHistory();
                $order_history->id_order = $order_id;
                $order_history->changeIdOrderState($id_cancel_order_state, $order_id);
                $order_history->addWithemail(true, array(
                    'order_name' => $order_state['reference'],
                ));
            } catch (Exception $e) {
                InbankLogs::addLog(
                    $order_id,
                    $this->module->l('Failed to change order status to cancelled.') . ' ' . $e->getMessage(),
                    InbankLogs::TYPE_WARNING,
                    '',
                    'CancelFrontController'
                );
            }
        }

        $details->last_state = 'cancelled';
        $details->timestamp = $timestamp;
        $details->save();

        // reloading customer into cookies because POST request is made from Inbank ePOS
        // on POST requests on CORS cookie is not loaded by the browser
        if (!isset($this->context->customer->id)
            || empty($this->context->customer->id)
            || $order->id_customer != $this->context->customer->id
        ) {
            $customer = new Customer($order->id_customer);
            $this->context->customer = $customer;
            $this->context->cookie->id_customer = (int)$customer->id;
            $this->context->cookie->customer_lastname = $customer->lastname;
            $this->context->cookie->customer_firstname = $customer->firstname;
            $this->context->cookie->passwd = $customer->passwd;
            $this->context->cookie->logged = 1;
            $this->context->cookie->id_lang = $order->id_lang;
            $this->context->cookie->id_currency = $order->id_currency;
            $this->context->cookie->id_guest = $customer->id_guest;
            $this->context->cookie->is_guest = $customer->isGuest();
            $this->context->currency = new Currency($order->id_currency);
            $this->context->language = new Language($order->id_lang);
            $this->context->cookie->email = $customer->email;

            if (method_exists($this->context->cookie, 'registerSession'))
                $this->context->cookie->registerSession(new CustomerSession());

            $this->context->cookie->write();
        }
        
        Tools::redirect($this->context->link->getPageLink('cart'));
    }
}
