<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

/**
 * @property Inbank $module
 */
class InbankPaymentModuleFrontController extends ModuleFrontController
{

    /**
     * Add CSS and JS files here
     */
    public function setMedia()
    {
        parent::setMedia();
        $this->addCSS(INBANK_MODULE_DIR.'views/css/'.$this->module->name.'.css');
    }

    /**
     * Controller initialization
     */
    public function init()
    {
        parent::init();

        $this->display_column_left = false;
        $this->display_column_right = false;

        $this->template = $this->getTemplatePath('payment.tpl');
        $this->ssl = true;
    }

    /**
     * @see FrontController::initContent()
     */
    public function initContent()
    {
        parent::initContent();

        $cart = $this->context->cart;
        $id_payment = (int)Tools::getValue('id_payment');
        $payment = new InbankPayment($id_payment, $this->context->language->id);

        if (!$this->module->checkCurrency($cart) || !Validate::isLoadedObject($payment)) {
            Tools::redirect('index.php?controller=order');
        }

        /**
         * @var $inbank_product AbstractPayment|InbankBalticsProduct
         */
        $inbank_product = new $payment->class_name($payment, $this->module);
        $order_total = $cart->getOrderTotal(true, Cart::BOTH);

        $placeholders = [];
        $payment_template = $inbank_product->getPaymentTemplate($order_total, [], $placeholders);

        $this->context->smarty->assign(array(
            'nb_products' => $cart->nbProducts(),
            'id_payment' => $payment->id,
            // we pass the title over from the payment selection page so we don't need to generate title again.
            'payment_title' => $inbank_product::replacePlaceholders($payment->custom_title, $placeholders),
            'payment_image' => $payment->use_logo ? $inbank_product->getLogoPath(true) : false,
            /* TODO: at the moment the page content is regenerated at this step again.
                we either need not to generate the template in payment selection step and only generate it here
                or cache the generated template somewhere and use it here.*/
            'payment_content' => $payment_template,
        ));
    }
}
