<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

/**
 * @property Inbank $module
 */
class InbankValidationModuleFrontController extends ModuleFrontController
{
    /**
     * @see FrontController::initContent()
     */
    public function initContent()
    {
        parent::initContent();

        $customer_owns_cart = false;

        //if order reference is set, order is already validated
        if (Tools::getIsset('reference')) {
            $id_order = InbankTools::getOrderIdByReference(Tools::getValue('reference'));
            $cart = Cart::getCartByOrderId($id_order);
            if ($this->context->customer->id == $cart->id_customer) {
                //marking that order is valid and we don't need to validate it
                $customer_owns_cart = true;
            }
        } else {
            $cart = $this->context->cart;
        }

        $customer = new Customer($cart->id_customer);
        //if cart has no customer id or address, order may already be present.
        if (
            $cart->id_customer == 0
            || $cart->id_address_delivery == 0
            || $cart->id_address_invoice == 0
            || !$this->module->active || !Validate::isLoadedObject($customer)
        ) {
            //cart has no customer or address, checking if an order is already present
            if (!$customer_owns_cart) {
                // no order present, redirect
                Tools::redirect($this->context->link->getPageLink('order', null, null, array('step' => 1)));
            }
        }

        // Check that this payment option is still available
        // in case the customer changed his address
        // just before the end of the checkout process
        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == $this->module->name) {
                $authorized = true;
                break;
            }
        }

        if (!$authorized) {
            $this->errors[] = $this->module->l('This payment method is not available.', 'validation');
        }

        // there are errors with this order, do not proceed
        if (!empty($this->errors)) {
            InbankLogs::addLog(
                0,
                'There were some errors trying to validate order before payment. ',
                InbankLogs::TYPE_ERROR,
                '',
                'ValidationFrontController',
                implode(',', $this->errors)
            );
            Tools::redirect($this->context->link->getPageLink('order', null, null, array('step' => 3)));
        }


        $currency = $this->context->currency;
        $total = (float)$cart->getOrderTotal(true, Cart::BOTH);
        $total = number_format((float)$total, 2, '.', '');

        $id_payment = Tools::getValue("inbank_payment");
        $payment = new InbankPayment($id_payment);
        /**
         * @var $inbank_product InbankBalticsProduct|AbstractPayment
         */
        $inbank_product = new $payment->class_name($payment, $this->module);

        $id_state = (int)Configuration::get('INBANK_PENDING');
        $title = $this->module->displayName;
        try {
            $title = $inbank_product::getDefaultTitle();
        } catch (Exception $e) {

        }
        // order is not validated and cart is valid. Validate order and set order state as pending.
        if (!$customer_owns_cart) {
            $mailVars = array();
            $this->module->validateOrder(
                (int)$cart->id,
                $id_state,
                $total,
                $title,
                null,
                $mailVars,
                (int)$currency->id,
                false,
                $customer->secure_key
            );
        }

        // Set order id
        $id_order = (int)$this->module->currentOrder;
        $inbank_product->handlePayment($id_order);
    }
}
