<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

require_once(dirname(__FILE__) . '/vendor/autoload.php');

if (!defined('_PS_VERSION_')) {
    exit;
}

if (!defined('INBANK_MODULE_DIR')) {
    define('INBANK_MODULE_DIR', _PS_MODULE_DIR_ . 'inbank/');
}

if (!defined('INBANK_IMG_DIR')) {
    define('INBANK_IMG_DIR', _PS_IMG_DIR_ . 'inbank/');
}

if (!defined('INBANK_MODULE_URL')) {
    define('INBANK_MODULE_URL', Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/inbank/');
}

if (!defined('INBANK_IMG_URL')) {
    define('INBANK_IMG_URL', Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'img/inbank/');
}

class Inbank extends PaymentModule
{
    public $_cachePath = "";
    public $is17 = true;
    public $is_cache_writable;

    public function __construct()
    {
        $this->name = 'inbank';
        $this->tab = 'payments_gateways';
        $this->version = '2.2.4';
        $this->author = 'Inbank';
        $this->module_key = '';
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';
        $this->bootstrap = 1;

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);

        parent::__construct();

        $this->displayName = $this->l('Pay Later Platform by Inbank');
        $this->description = $this->l('Increase your revenue by turning visitors into customers. Pay later payment solutions from Inbank make your goods more affordable for your customers whilst you are still selling at full price.');
        $this->confirmUninstall = $this->l('Would you like to uninstall Pay Later Platform by Inbank?');
        $this->is17 = Tools::version_compare(_PS_VERSION_, "1.7", ">");
        $this->_cachePath = INBANK_MODULE_DIR . "cache/";
        $this->is_cache_writable = is_writable($this->_cachePath);
    }

    public function install()
    {
        $res = parent::install();
        try {
            $installer = new InbankInstaller($this);
            $res &= $installer->runInstaller();
        } catch (Exception $e) {
            $this->_errors[] = sprintf($this->l('An error occurred during installation process: %s'), get_class($e) . ': ' . $e->getMessage());

            $this->uninstall();

            return false;
        }

        return $res;
    }


    public function uninstall()
    {
        $installer = new InbankInstaller($this);
        $res = $installer->runUninstaller();

        return parent::uninstall() && $res;
    }

    public function getContent()
    {
        if(Tools::isSubmit('searchProducts') && Tools::isSubmit('q')) {
            $search_query = pSQL(Tools::getValue('q'));
            die(implode("\n", $this->searchProducts($search_query)));
        }
        $output = '';

        // Checking if configuration file is removed
        $config_file_dir = INBANK_MODULE_DIR.'configuration.json';
        if (is_file($config_file_dir)) {
            unlink($config_file_dir);
            if (is_file($config_file_dir)) {
                $output .= $this->displayError($this->l('Warning! There was a problem trying to remove configuration file while installing. Please delete the file manually in module root directory'));
            }
        }

        // Check for available updates (only version check)
        if (!class_exists('InbankUpdater')) {
            require_once(dirname(__FILE__) . '/src/Helpers/InbankUpdater.php');
        }
        if (!$this->is_cache_writable) {
            $output .= $this->displayError('Warning! There seems to be a problem writing to cache ('.$this->_cachePath.') folder');
        }

        // Set confirmation / alert if exist;
        $output .= $this->getRedirectAlerts();

        if (Tools::isSubmit('updateinbank_general_settings')) {
            $this->updateGeneralSettings();

            $output .= $this->displayConfirmation($this->l('Global settings updated'));
        }

        if (Tools::isSubmit('add_inbank_payment')) {
            $this->addInbankPayment();
        }
        else if (Tools::isSubmit("updateinbank_payment"))
        {
            //inbank payment update form
            return $output . $this->updateInbankPaymentForm();
        }
        else if (Tools::isSubmit('statusinbank_payment'))
        {
            $this->updateInbankPaymentStatus();
        }
        else if (Tools::isSubmit('deleteinbank_payment'))
        {
            $this->deleteInbankPayment();
        }
        else if (Tools::isSubmit("updateInbankPayment"))
        {
            $this->updateInbankPayment();
        } else if (Tools::isSubmit("newInbankPayment"))
        {
            $content = $this->newInbankPaymentForm();
        } else if (Tools::isSubmit('checkOrderState'))
        {
            $this->checkOrderState();
        } else if (Tools::isSubmit('updateAllProducts'))
        {
            $scanned_directory = array_diff(scandir($this->_cachePath), array('..', '.', 'index.php'));
            foreach ($scanned_directory as $file) {
                unlink($this->_cachePath.$file);
            }
            $this->updateAllProducts();
            $this->setRedirectAlert($this->l('Payment methods updated successfully.'), 'success');
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } else {
            $content = InbankUpdater::check('inbank', $this->version, _PS_VERSION_);
            $content .= $this->generateList();

            $this->context->smarty->assign([
                'generalSettings' => $this->generateGlobalSettings(),
                'new_payment_link' => $this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&newInbankPayment=1',
            ]);
        }

        $this->context->smarty->assign(
            array(
                'module_dir' => $this->_path,
                'content' => $content,
            )
        );

        $logo = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/logo.tpl');
        $output .= $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $logo . $output;
    }

    private function searchProducts($q)
    {
        $products = Product::searchByName($this->context->language->id, $q, null, 20);

        if(empty($products)) {
            return [];
        }

        $search_results = [];

        foreach ($products as $product) {
            $search_results[] = json_encode([
                'id_product' => $product['id_product'],
                'name' => $product['name'],
                'reference' => $product['reference']
            ]);
        }

        return $search_results;
    }

    private function updateGeneralSettings()
    {
        $update_values = ['INBANK_COMPLETED', 'INBANK_PENDING', 'INBANK_CANCELLED'];
        foreach ($update_values as $name) {
            if (Tools::getIsset($name)) {
                $value = Tools::getValue($name);
                Configuration::updateValue($name, $value);
            }
        }
    }

    private function addInbankPayment()
    {
        $payment = new InbankPayment();
        $class_name = Tools::getValue("class_name");
        if ($class_name::saveFormValues($payment)) {
            $this->getProductInfo($payment);
        }

        //resetting placeholder values

        /*
        if (is_array($payment->custom_title)) {
            foreach ($payment->custom_title as $id_lang => $title) {
                if (!empty($title) && $title == $payment->title[$id_lang]) {
                    $payment->custom_title[$id_lang] = '';
                }
            }
        } else {
            if (empty($payment->custom_title) && $payment->custom_title == $payment->title) {
                $payment->custom_title = '';
            }
        }

        if (is_array($payment->custom_additional_details)) {
            foreach ($payment->custom_additional_details as $id_lang => $detail) {
                if (!empty($detail) && $detail == $payment->additional_details[$id_lang]) {
                    $payment->custom_additional_details[$id_lang] = '';
                }
            }
        } else {
            if (empty($payment->custom_additional_details) && $payment->custom_additional_details == $payment->additional_details) {
                $payment->custom_additional_details = '';
            }
        }*/

        $payment->save();

        $result = InbankPaymentFormHelper::uploadImage($payment->id_payment, 'custom_logo');
        if ($result !== true)
        {
            if (is_array($result))
                $this->setRedirectAlert(implode(';', $result), 'error');
            else
                $this->setRedirectAlert($this->l('Error uploading custom logo.'), 'error');

            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&add_inbank_payment=1');
        }

        $this->setRedirectAlert($this->l('New payment method created.'), 'success');

        Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
    }

    private function updateInbankPaymentForm()
    {
        $id_payment = (int)Tools::getValue("id_payment");

        if (Tools::getIsset('deletecustomlogo')) {
            @unlink(INBANK_IMG_DIR . $id_payment . '_small.jpg');
            @unlink(INBANK_IMG_DIR . $id_payment . '_medium.jpg');
        }

        $payment_form = new InbankPaymentForm($this);
        //setting up js for hiding test/live fields via test/live switch
        Media::addJsDef([
            'product_search_link' => $this->context->link->getAdminLink(
                'AdminModules',
                true,
                [],
                [
                    'configure' => $this->name,
                    'searchProducts' => 1
                ]
            )
        ]);
        $this->context->controller->addJS($this->_path . "views/js/inbank_back_edit.js");
        return $payment_form->updateProductForm($id_payment);
    }

    private function updateInbankPaymentStatus() {
        $id_payment = (int)Tools::getValue('id_payment');
        $paymentModel = new InbankPayment($id_payment);
        $paymentModel->enabled = !$paymentModel->enabled;
        $paymentModel->save();
        $this->setRedirectAlert($this->l('Payment status updated successfully.'), 'success');
        Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
    }

    private function deleteInbankPayment() {
        $id_payment = (int)Tools::getValue("id_payment");
        $paymentModel = new InbankPayment($id_payment);
        if ($paymentModel->delete()) {
            $files = glob(INBANK_MODULE_DIR.'cache/'.$id_payment.'_*.html');
            foreach ($files as $file)
            {
                if (!in_array(basename($file), ['.', '..', 'index.php']))
                    @unlink($file);
            }

            // deleting custom logo if exists
            if (file_exists(INBANK_IMG_DIR.$id_payment.'_small.jpg'))
                @unlink(INBANK_IMG_DIR.$id_payment.'_small.jpg');

            if (file_exists(INBANK_IMG_DIR.$id_payment.'_medium.jpg'))
                @unlink(INBANK_IMG_DIR.$id_payment.'_medium.jpg');

            $this->setRedirectAlert($this->l('Payment deleted successfully.'), 'success');
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } else {
            $this->setRedirectAlert($this->l('Failed to delete payment.'), 'error');
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }
    }

    private function updateInbankPayment() {
        if (Tools::isSubmit("id_payment")) {
            $paymentModel = new InbankPayment((int)Tools::getValue("id_payment"));
            $class_name = $paymentModel->class_name;
            $product_code = Tools::getValue('product_code');
            $product_code_debug = Tools::getValue('product_code_debug');
            $show_success = true;
            if (!empty($product_code) || !empty($product_code_debug)) {
                $check_code = empty($product_code) ? (string)$product_code_debug : (string)$product_code;
                $exists = InbankPaymentFormHelper::checkForDuplicate(
                    $check_code,
                    $paymentModel->id_country,
                    $paymentModel->class_name,
                    $paymentModel->id
                );
                if ($exists) {
                    $show_success = false;
                    $product_link = $this->context->link->getAdminLink(
                        'AdminModules',
                        true,
                        [],
                        [
                            'configure' => $this->name,
                            'module_name' => $this->name,
                            'tab_module' => 'payments_gateways',
                            'updateinbank_payment' => 1,
                            'id_payment' => $exists
                        ]
                    );
                    unset($_POST['product_code']);
                    unset($_POST['product_code_debug']);
                    $this->setRedirectAlert(
                        $this->l('Payment option with this product code already exists.').
                        ' <a href="'.$product_link.'">'.$this->l('You can check the product here').'</a>',
                        'error'
                    );
                }
            }
            //$languages = Language::getLanguages();
            //unset custom_fields if they're matching original
            /*
            $fields = ['additional_details' => 'custom_additional_details', 'title' => 'custom_title'];
            foreach ($languages as $lang) {
                $id_lang = $lang['id_lang'];
                foreach ($fields as $original => $custom_field) {
                    $field = $custom_field.'_'.$id_lang;
                    $details = Tools::getValue($field);
                    if ($paymentModel->$original[$id_lang] === $details) {
                        unset($_GET[$field]);
                        unset($_POST[$field]);
                    }
                }
            }
            */
            if ($class_name::saveFormValues($paymentModel)) {
                $this->getProductInfo($paymentModel);
                $restrictionForm = new InbankRestrictionForm($paymentModel, $this, $this->context);
                $restrictionForm->saveRestrictions();
                $result = InbankPaymentFormHelper::uploadImage($paymentModel->id_payment, 'custom_logo');

                if ($result !== true)
                {
                    if (is_array($result))
                        $this->setRedirectAlert(implode(';', $result), 'error');
                    else
                        $this->setRedirectAlert($this->l('Error uploading custom logo.'), 'error');

                    Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&updateinbank_payment=1' . '&id_payment=' . $paymentModel->id);
                }
            }

            if ($show_success) {
                $this->setRedirectAlert($this->l('Payment settings updated.'), 'success');
            }

            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } else {
            $this->setRedirectAlert($this->l('Product not found.'), 'error');
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }
    }

    private function newInbankPaymentForm() {
        $form_helper = new InbankPaymentForm($this);
        $this->context->controller->addJS($this->_path . "views/js/inbank_back.js");
        Media::addJsDef([
            'filter_payments_link' => $this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&newInbankPayment=1&new_payment_action=get_available_payments',
        ]);
        return $form_helper->newProductForm();
    }

    private function checkOrderState() {
        $id_order = (int)Tools::getValue('id_order');
        if (Tools::isSubmit('confirm_order') || Tools::isSubmit('cancel_order')) {
            $details = InbankDetail::getDetailsByOrderId($id_order);
            $payment = new InbankPayment($details->id_payment);
            $inbank_product = new $payment->class_name($payment, $this);
            if (!empty($details->contract_uuid)) {
                if (Tools::isSubmit('confirm_order')) {
                    $inbank_product->api->approveContract($details->contract_uuid);
                } else {
                    if (Tools::isSubmit('cancel_order')) {
                        $inbank_product->api->cancelContract($details->contract_uuid);
                    }
                }
                //Tools::redirect($this->context->link->getAdminLink('AdminOrders', true, [], ['id_order' => $id_order, 'vieworder' => 1]));
            }
        }

        if ($id_order > 0) {
            $order_obj = new Order($id_order);
            $details = InbankDetail::getDetailsByOrderId($id_order);
            $payment = new InbankPayment($details->id_payment);
            if (empty($payment->class_name)) {
                Tools::redirect($this->context->link->getAdminLink('AdminOrders', true, [], ['id_order' => $id_order, 'vieworder' => 1]));
            }
            $inbank_product = new $payment->class_name($payment, $this);
            if ($payment->needs_approval && !empty($details->contract_uuid)) {
                $contract_details = $inbank_product->api->getContractDetails($details->contract_uuid);
                if (isset($contract_details->errors)) {
                    InbankLogs::addLog($id_order, implode($contract_details->erors));
                } else {
                    $details->timestamp = time();
                    $details->last_state = $contract_details->contract->status;
                    $details->save();

                    if ($contract_details->contract->status == 'activated') {
                        $local_state = (int)Configuration::get('INBANK_COMPLETED');
                    } else {
                        $local_state = (int)Configuration::get('INBANK_'.Tools::strtoupper($contract_details->contract->status));
                    }
                    if ($local_state && $order_obj->current_state != $local_state) {
                        $order_history = new OrderHistory();
                        $order_history->id_order = $id_order;
                        $order_history->id_order_state = $local_state;
                        $order_history->add();
                    }
                    Tools::redirectAdmin($this->context->link->getAdminLink('AdminOrders', true, ['id_order' => $id_order], ['id_order' => $id_order, 'vieworder' => 1]));
                }
            }
            if ($details->session_uuid) {
                $session_details = $inbank_product->api->getSessionDetails($details->session_uuid);
                if (isset($session_details->errors)) {
                    InbankLogs::addLog($id_order, implode(',', $session_details->errors));
                } else {
                    $details->timestamp = time();
                    $details->last_state = $session_details->status;
                    $details->contract_uuid = $session_details->credit_contract_uuid;
                    $details->save();

                    $local_state = (int)Configuration::get('INBANK_' . Tools::strtoupper($session_details->status));
                    if ($local_state && $order_obj->current_state != $local_state) {
                        $order_history = new OrderHistory();
                        $order_history->id_order = $id_order;
                        $order_history->id_order_state = $local_state;
                        $order_history->add();
                    }
                }
            }
        }

        Tools::redirectAdmin($this->context->link->getAdminLink('AdminOrders', true, ['id_order' => $id_order], ['id_order' => $id_order, 'vieworder' => 1]));
    }

    private function generateList()
    {
        $this->context->controller->addJS(INBANK_MODULE_DIR.'views/js/inbank_back_status_btn.js');
        Media::addJsDef([
            'inbank' => [
                'trans' => ['status_btn' => $this->l('Disable/Enable')],
                'links' => [
                    'status_btn' => $this->context->link->getAdminLink(
                        'AdminModules',
                        true,
                        [],
                        [
                            'configure' => $this->name,
                            'tab_module' => 'payments_gateways',
                            'module_name' => $this->name,
                            'statusinbank_payment' => 1
                        ]
                    )
                ]
            ],
        ]);
        $list = Db::getInstance()->executeS(
            "SELECT a.id_payment, a.class_name, COALESCE(NULLif (a.product_code, ''), a.product_code_debug) AS product_code, a.enabled, b.*, cl.name AS country_name,
            COALESCE(NULLif (b.custom_title, ''), b.title) AS title
            FROM " . _DB_PREFIX_ . "inbank_payment a
            LEFT JOIN " . _DB_PREFIX_ . "inbank_payment_lang b ON a.id_payment = b.id_payment
            LEFT JOIN " . _DB_PREFIX_ . "country c ON c.id_country = a.id_country
            LEFT JOIN " . _DB_PREFIX_ . "country_lang cl ON cl.id_country = c.id_country
            WHERE b.id_lang = " . $this->context->language->id . " AND cl.id_lang = " .$this->context->language->id
        );

        // protection from missing class
        foreach ($list as $key => $row)
        {
            if (!class_exists($row['class_name']))
                unset($list[$key]);
        }

        $field_list = array(
            //'id_payment' => array('title' => $this->l('ID'), 'align' => 'center', 'search' => false),
            'class_name' => array(
                'title' => $this->l('Product'),
                'align' => 'center',
                'search' => false,
                'callback' => 'formatProductCode',
                'callback_object' => 'Inbank',
            ),
            'product_code' => array('title' => $this->l('Product code'), 'align' => 'center', 'search' => false),
            'country_name' => array('title' => $this->l('Country'), 'align' => 'center', 'search' => false),
            'title' => array('title' => $this->l('Payment title'), 'align' => 'center', 'search' => false),
            //'id_shop' => array('title' => $this->l('Shop'), 'align' => 'center', 'search' => false),
            'enabled' => array(
                'title' => $this->l('Status'),
                'align' => 'center',
                'active' => 'status',
                'type' => 'bool',
                'search' => false,
            ),
        );
        $helperList = new HelperList();
        $helperList->shopLinkType = '';
        $helperList->simple_header = false;
        $helperList->actions = array("edit", "delete");
        $helperList->show_toolbar = false;
        $helperList->module = $this;
        $helperList->listTotal = count($list);
        $helperList->identifier = 'id_payment';
        $helperList->title = $this->l("Inbank Payment Methods");
        $helperList->table = "inbank_payment";
        $helperList->token = Tools::getAdminTokenLite('AdminModules');
        $helperList->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;

        return $helperList->generateList($list, $field_list);
    }

    private function generateGlobalSettings()
    {
        $order_statuses = OrderState::getOrderStates($this->context->language->id);

        $cron_link = $this->context->link->getModuleLink($this->name, 'cron', ['inbank_token' => $this->generateHash($this->name)]);

        $form_fields = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('General Inbank settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'select',
                        'label' => $this->l('Confirmed order state'),
                        'desc' => $this->l('Select which state to use when financing is confirmed by Inbank'),
                        'name' => 'INBANK_COMPLETED',
                        'options' => array(
                            'query' => $order_statuses,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Pending order state'),
                        'desc' => $this->l('Select which state to use when financing is in pending state'),
                        'name' => 'INBANK_PENDING',
                        'options' => array(
                            'query' => $order_statuses,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Canceled order state'),
                        'desc' => $this->l('This state will be used when the payment is canceled by the user or by Inbank'),
                        'name' => 'INBANK_CANCELLED',
                        'options' => array(
                            'query' => $order_statuses,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'html',
                        'name' => 'html_data',
                        'label' => $this->l('Cron link'),
                        'desc' => $this->l(
                            'By default, products are updated automatically every 24 hours. Use this cron job if you\'d like to manually update products at your chosen time.'
                        ),
                        'html_content' => '<label>'.$cron_link.'</label>'
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
                'buttons' => array(
                    array(
                        'href' => $this->context->link->getAdminLink('AdminModules') . '&configure=' . $this->name . '&tab_module' . $this->tab . '&module_name=' . $this->name . '&updateAllProducts=true',
                        'title' => $this->l('Update products'),
                        'icon' => 'process-icon-refresh',
                    ),
                ),
            ),
        );

        $field_values = [];
        $field_values['INBANK_COMPLETED'] = Configuration::get('INBANK_COMPLETED');
        $field_values['INBANK_PENDING'] = Configuration::get('INBANK_PENDING');
        $field_values['INBANK_CANCELLED'] = Configuration::get('INBANK_CANCELLED');

        $form = new InbankPaymentForm($this);

        return $form->buildPaymentForm($form_fields, $field_values, 'updateinbank_general_settings');
    }

    public static function formatProductCode($class_name)
    {
        try {
            return $class_name::getDefaultTitle();
        } catch (Exception $e) {
            return $class_name;
        }
    }

    /**
     * @param $payment InbankPayment;
     */
    private function getProductInfo($payment)
    {
        if (Validate::isLoadedObject($payment)) {
            if ((!empty($payment->api_key) && !empty($payment->shop_uuid)) || (!empty($payment->api_key_debug) && !empty($payment->shop_uuid_debug))) {
                $class_name = $payment->class_name;
                /**
                 * @var $product AbstractPayment|InbankBalticsProduct
                 */
                $product = new $class_name($payment, $this);
                $product_info = $product->api->getInfo();
                if (isset($product_info->code) && $product_info->code == $product->getProductCode()) {
                    if ($product_info->subtype !== $product->subtype) {
                        $this->setRedirectAlert($this->l('Your selected Product subtype doesn\'t match the supplied product code.'), 'error');
                        Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&updateinbank_payment=1' . '&id_payment=' . $payment->id);
                    }
                    //updates payment values from api and generates templates
                    $product->updateSelf(true, $product_info);
                } else {
                    $payment->enabled = 0;
                    $payment->save();
                    if (isset($product_info->error)) {
                        $this->setRedirectAlert('There were some errors trying to get info from Inbank API: ' . implode(',', $product_info->error), 'error');
                        Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&updateinbank_payment=1' . '&id_payment=' . $payment->id);
                    }
                }
            }
        }
    }

    // used to display leasing options above add to cart button
    public function hookDisplayProductPriceBlock($params)
    {
        if (isset($params['type']) && $params['type'] == 'price') {
            try {
                InbankBalticsProduct::updatePaymentData($this);
            } catch (Exception $e) {
                InbankLogs::addLog(0, 'Failed to check for API updates.');
            }
            if ($this->is17) {
                return $this->getPaymentMethods17($params, 'displayProductPriceBlock');
            } else {
                return $this->getPaymentMethods16($params, 'displayProductPriceBlock');
            }
        }
    }

    private function getPaymentMethods16($params, $hook_name)
    {
        if (Tools::getValue('controller') !== 'product') {
            return;
        }
        if (isset($params['type']) && $params['type'] == 'price') {
            $product = $params['product'];

            if (isset($product->tax_rate)) {
                $price = round($product->price * (($product->tax_rate / 100) + 1), 2);
            } else {
                $price = round($product->price, 2);
            }
        } else if (isset($params['product'])) {
            $product = $params['product'];
            if (isset($product->tax_rate)) {
                $price = round($product->price * (($product->tax_rate / 100) + 1), 2);
            } else {
                $price = round($product->price, 2);
            }
        }

            //getting all eligible payment methods
            $methods = InbankPayment::getEligiblePaymentMethods($price, null, null, $hook_name, 1, null, $product->id);
            $payments = "";
            if (!empty($methods)) {
                foreach ($methods as $method) {
                    $class_name = $method->class_name;
                    if (method_exists($class_name, 'getAvailableCurrencies')) {
                        $product_available_currencies = $class_name::getAvailableCurrencies();
                        $context_currency = $this->context->currency->iso_code;
                        if (!in_array($context_currency, $product_available_currencies)) {
                            continue;
                        }
                    }
                    $inbank_product = new $class_name($method, $this);
                    $payments .= $inbank_product->getCalcSmallTemplate($price);
                }
            }

            return $payments;
    }

    private function getPaymentMethods17($params, $hook_name)
    {
        if (isset($params['product'])) {
            $product = $params['product'];
            $price = $product->price_amount;
        } else {
            $id_product = (int)Tools::getValue('id_product');
            $id_product_attribute = Tools::getValue('id_product_attribute', null);
            $price = Product::getPriceStatic($id_product, true, $id_product_attribute, 2);
        }

        if (isset($price) && $price > 0) {
            $methods = InbankPayment::getEligiblePaymentMethods(
                $price,
                null,
                null,
                $hook_name,
                1,
            null,
                isset($product->id) ? $product->id : $id_product
            );
            $payments = "";
            if (!empty($methods)) {
                foreach ($methods as $method) {
                    $class_name = $method->class_name;
                    if (method_exists($class_name, 'getAvailableCurrencies')) {
                        $product_available_currencies = $class_name::getAvailableCurrencies();
                        $context_currency = $this->context->currency->iso_code;
                        if (!in_array($context_currency, $product_available_currencies)) {
                            continue;
                        }
                    }
                    $inbank_product = new $class_name($method, $this);
                    $payments .= $inbank_product->getCalcSmallTemplate($price);
                }
            }

            return $payments;
        }
    }

    private function convertCurrency($price, $class_name) {
        $context_currency = $this->context->currency;
        if (method_exists($class_name, 'getAvailableCurrencies')) {
            $product_available_currencies = $class_name::getAvailableCurrencies();
            if (!in_array($context_currency, $product_available_currencies)) {
                $product_currency_iso = reset($product_available_currencies);
                $product_currency = Currency::getIdByIsoCode($product_currency_iso);
                $product_currency = new Currency($product_currency);

                return Tools::convertPriceFull($price, $context_currency, $product_currency);
            }
        }
        return $price;
    }

    public function hookAdminOrder($params)
    {
        return;
        $order = new Order($params['id_order']);
        $needs_action = false;

        if ($order->module == $this->name) {
            $details = InbankDetail::getDetailsByOrderId($order->id);
            if (!empty($details)) {
                $date_time = new DateTime();
                $date_time->setTimestamp($details->timestamp);
                $details->timestamp = $date_time->format('Y-m-d H:i:s');
            }
            if ($order->current_state == Configuration::get('INBANK_GRANTED')) {
                $needs_action = 1;
            }
            $payment = new InbankPayment($details->id_payment);
            if ($payment && $payment->needs_approval) {
                $details->needs_approval = $payment->needs_approval;
            }

            $this->context->smarty->assign(
                array(
                    'details' => $details,
                    'module_link' => $this->context->link->getAdminLink('AdminModules', true, [], ['configure' => $this->name, 'checkOrderState' => 1, 'id_order' => $order->id]),
                    'needs_action' => $needs_action,
                )
            );

            if (version_compare(_PS_VERSION_, '1.7.7') >= 1) {
                $output = $this->context->smarty->fetch($this->local_path . 'views/templates/hook/order_177.tpl');
            } else {
                $output = $this->context->smarty->fetch($this->local_path . 'views/templates/hook/order.tpl');
            }

            return $output;
        }
    }

    public function hookDisplayHeader()
    {
        if (Tools::getValue('controller') == 'product' || Tools::getValue('controller') == 'category') {
            if ($this->is17) {
                $this->context->controller->addCSS($this->_path.'views/css/inbank_front.css');
            } else {
                $this->context->controller->addCSS($this->_path.'views/css/inbank_front_16.css');
            }
            $this->context->controller->addJS($this->_path . 'views/js/inbank_front.js');
            Media::addJsDef([
                'inbank' => [
                    'image_link' => _MODULE_DIR_ . $this->name . '/views/img/svg/small/inbank-logo.svg'
                ]
            ]);
        }
    }

    public function hookDisplayLeasing($params)
    {
        if ($this->is17) {
            return $this->getPaymentMethods17($params, 'displayLeasing');
        } else {
            return $this->getPaymentMethods16($params, 'displayLeasing');
        }
    }

    public function hookdisplayProductAdditionalInfo($params)
    {
        if ($this->is17) {
            return $this->getPaymentMethods17($params, 'displayProductAdditionalInfo');
        } else {
            return $this->getPaymentMethods16($params, 'displayProductAdditionalInfo');
        }
    }

    public function hookDisplayReassurance($params)
    {
        if ($this->is17) {
            return $this->getPaymentMethods17($params, 'displayReassurance');
        } else {
            return $this->getPaymentMethods16($params, 'displayReassurance');
        }
    }

    public function hookDisplayProductDeliveryTime($params)
    {
        if ($this->is17) {
            return $this->getPaymentMethods17($params, 'displayProductDeliveryTime');
        } else {
            return $this->getPaymentMethods16($params, 'displayProductDeliveryTime');
        }
    }

    public function hookPayment($params)
    {
        if (!$this->active) {
            return '';
        }
        if (!$this->checkCurrency($params['cart'])) {
            return '';
        }

        try {
            InbankBalticsProduct::updatePaymentData($this);
        } catch (Exception $e) {
            InbankLogs::addLog(0, 'Failed to check for API updates.');
        }

        // Check minimum cart price
        $minimal_cart_price = Configuration::get('INBANK_MINIMUM');
        $max_cart_price = Configuration::get('INBANK_MAXINUM');
        $cartObj = new Cart((int)$params['cart']->id);
        $total = $cartObj->getOrderTotal(true, Cart::ONLY_PRODUCTS);

        if ($minimal_cart_price > $total || $total > $max_cart_price) {
            return '';
        }

        $this->smarty->assign(array(
            'this_path' => $this->_path,
            'total_cart' => $params['cart']->getOrderTotal(true, Cart::BOTH_WITHOUT_SHIPPING),
            'minimal_cart_price' => $minimal_cart_price,
            'max_cart_price' => $max_cart_price,
            'this_path_bw' => $this->_path,
            'this_path_ssl' => INBANK_MODULE_URL,
        ));

        return $this->display(__FILE__, 'payment.tpl');
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active || !$this->checkCurrency($params['cart'])) {
            return;
        }

        try {
            InbankBalticsProduct::updatePaymentData($this);
        } catch (Exception $e) {
            InbankLogs::addLog(0, 'Failed to check for API updates.');
        }

        $cart = $params['cart'];
        $total_sum = $cart->getOrderTotal();
        $id_country = Address::getCountryAndState($cart->id_address_delivery)['id_country'];
        $currency = new Currency($cart->id_currency);

        try {
            $payments = InbankPayment::getEligiblePaymentMethods(
                $total_sum,
                $id_country,
                null,
                null,
                null,
                null,
                array_column($cart->getProducts(), 'id_product')
            );
        } catch (Exception $e) {
            InbankLogs::addLog(0, "Failed to get eligible payment options. " . $e->getMessage(), InbankLogs::TYPE_ERROR);
            $payments = [];

            return false;
        }

        if (empty($payments)) {
            return false;
        }

        $payment_options = [];
        foreach ($payments as $payment) {
            try {
                $class_name = $payment->class_name;
                $inbank_product = new $class_name($payment, $this);
            } catch (Exception $e) {
                break;
            }
            $new_option = $inbank_product->displayPaymentOption17($total_sum, $currency->iso_code);
            if ($new_option !== false) {
                $payment_options[] = $new_option;
            }
        }

        return $payment_options;
    }

    public function hookDisplayPayment($params)
    {
        if (!$this->active || !$this->checkCurrency($params['cart'])) {
            return;
        }

        try {
            InbankBalticsProduct::updatePaymentData($this);
        } catch (Exception $e) {
            InbankLogs::addLog(0, 'Failed to check for API updates.');
        }

        $cart = $params["cart"];
        if (!Validate::isLoadedObject($cart)) {
            return '';
        }

        $total_sum = $cart->getOrderTotal();
        $id_country = Address::getCountryAndState($cart->id_address_delivery)['id_country'];

        try {
            $inbank_products = InbankPayment::getEligiblePaymentMethods(
                $total_sum,
                $id_country,
                null,
                null,
                null,
                null,
                array_column($cart->getProducts(), 'id_product')
            );
        } catch (Exception $e) {
            InbankLogs::addLog(0, 'Something went wrong when trying to query eligible payment methods. ' . $e->getMessage());
            $inbank_products = [];
        }

        $options = "";
        if (!empty($inbank_products)) {
            foreach ($inbank_products as $product) {
                $payment_option = new $product->class_name($product, $this);
                $options .= $payment_option->displayPaymentOption16($total_sum, $this->context->currency->iso_code);
            }
        }

        return $options;
    }

    public function hookDisplayProductExtraContent($params) {
        return null;
        $price = $params['product']->price;
        $available_methods = InbankPayment::getEligiblePaymentMethods(
            $price,
            null,
            null,
            null,
            null,
            1
        );
        $calculators = '';
        foreach ($available_methods AS $available_method) {
            $class_name = $available_method->class_name;
            try {
                $available_currencies = $class_name::getAvailableCurrencies();
                $context_currency = $this->context->currency->iso_code;
                if (in_array($context_currency, $available_currencies)) {
                    $product = new $class_name($available_method, $this);
                    $calculators .= $product->getCalcLargeTemplate($price);
                }
            } catch (Exception $e) {

            }
        }
        $content = [];
        if (!empty($calculators)) {
            $extraContent = new \PrestaShop\PrestaShop\Core\Product\ProductExtraContent();
            $extraContent->setTitle('Inbank Hire Purchase');
            $extraContent->setContent($calculators);
            $extraContent->setAttr(['id' => 'inbank_hp_calculator', 'class' => 'inbank_hp_calculator']);
            $content[] = $extraContent;
        }
        return $content;
    }

    public function hookDisplayBackOfficeHeader()
    {
        if (Tools::getValue('controller') == 'AdminModules' && Tools::getValue('configure') == $this->name) {
            $this->context->controller->addCSS($this->_path . 'views/css/inbank.css');
        }
    }

    public function hookDisplayPaymentReturn() {
        $id_order = (int)Tools::getValue('id_order');
        if (!$id_order) {
            $id_cart = (int)Tools::getValue('id_cart');
            if ($this->is17) {
                $id_order = Order::getIdByCartId($id_cart);
            } else {
                $id_order = Order::getOrderByCartId($id_cart);
            }
        }
        if ($this->is17) {
            return $this->paymentReturn17($id_order);
        } else {
            return $this->paymentReturn16($id_order);
        }
    }

    private function paymentReturn17($id_order) {
        if (!$id_order) {
            return;
        }
        $order = Db::getInstance()->getRow('SELECT module FROM '._DB_PREFIX_.'orders WHERE id_order = '.(int)$id_order);
        if ($order['module'] != $this->name) {
            return;
        }
        $detail = InbankDetail::getDetailsByOrderId($id_order);
        if (empty($detail)) {
            return;
        }
        /*
        if (Tools::isSubmit('message')) {
            $message = json_decode(Tools::getValue('message'));
            if (!isset($message->uuid) && $message->uuid !== $detail->session_uuid) {
                return false;
            }
        } else {
            return false;
        }*/
        try {
            $payment = new InbankPayment($detail->id_payment, $this->context->language->id);
        } catch (Exception $e) {
            InbankLogs::addLog(
                $id_order,
                $this->l('Failed to load payment while trying to check for granted text.'),
                InbankLogs::TYPE_ERROR
            );
            return;
        }
        if (!empty($payment->granted) && $payment->needs_approval) {
            //first we can check if return controller already set state as granted
            $granted_state_id = (int)Configuration::get('INBANK_GRANTED');
            // second request is made to get newest data from DB as few concurrent requests may be performing the same action
            $current_state = (int)Db::getInstance()->getValue('SELECT current_state FROM '._DB_PREFIX_.'orders WHERE id_order = '.(int)$id_order);
            if ($current_state === $granted_state_id) {
                return $payment->granted;
            }

            /*
            //if not, we can check the current state by getting session details
            $class_name = $payment->class_name;
            /**
             * @var $inbank_product AbstractPayment|InbankBalticsProduct
             **
            $inbank_product = new $class_name($payment, $this);
            $response = $inbank_product->api->getSessionDetails($detail->session_uuid);
            if (isset($response->status) && $response->status == 'granted')
            {
                // second request is made to get newest data from DB as few concurrent requests may be performing the same action
                $current_state = (int)Db::getInstance()->getValue('SELECT current_state FROM '._DB_PREFIX_.'orders WHERE id_order = '.(int)$id_order);
                if ($current_state !== $granted_state_id) {
                    try {
                        $order_history = new OrderHistory();
                        $order_history->id_order = $id_order;
                        $order_history->id_order_state = $granted_state_id;
                        $order_history->add();

                        $detail->last_state = 'granted';
                        $detail->timestamp = Tools::getValue('timestamp');
                        $detail->save();
                    } catch (Exception $e) {
                        InbankLogs::addLog(
                            $id_order,
                            $this->l('Failed to set order status to granted.'),
                            InbankLogs::TYPE_ERROR,
                            '',
                            'PaymentReturn',
                            $e->getMessage()
                        );
                    }
                }
                return $payment->granted;
            }
            */
            return false;
        }
        return false;
    }

    private function paymentReturn16($id_order) {
        $status = 'ok';
        if (!$id_order) {
            $status = 'error';
        }
        $order = new Order($id_order);
        if ($order->module != $this->name) {
            $status = 'error';
        }
        $detail = InbankDetail::getDetailsByOrderId($id_order);
        if (empty($detail)) {
            $status = 'error';
        }
        $payment = new InbankPayment($detail->id_payment, $this->context->language->id);
        //check if payment is granted and has a message set
        $payment_granted_id = (int)Configuration::get('INBANK_GRANTED');
        if (!empty($payment->granted) && (int)$order->current_state === $payment_granted_id) {
            $this->context->smarty->assign(['status_text' => $payment->granted]);
        }

        $this->context->smarty->assign(['status' => $status]);

        return $this->context->smarty->fetch(INBANK_MODULE_DIR . 'views/templates/hook/payment_return.tpl');
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }

        return false;
    }

    public function displayWarning($warning)
    {
        if (version_compare(_PS_VERSION_, '1.6.1') >= 0) {
            $output = parent::displayWarning($warning);
        } else {
            $output = '
            <div class="bootstrap">
            <div class="module_warning alert alert-warning" >
			<button type="button" class="close" data-dismiss="alert">&times;</button>';

            if (is_array($warning)) {
                $output .= '<ul>';
                foreach ($warning as $msg) {
                    $output .= '<li>' . $msg . '</li>';
                }
                $output .= '</ul>';
            } else {
                $output .= $warning;
            }
            $output .= '</div></div>';
        }

        return $output;
    }

    private function multiLang($value)
    {
        $multilangField = [];
        $langs = Language::getLanguages(false, false, true);
        foreach ($langs as $id_lang) {
            $multilangField[$id_lang] = $value;
        }

        return $multilangField;
    }

    public function getAvailablePositions($form_builder = false)
    {
        $hooks = [
            'displayProductPriceBlock' => $this->l('After product price')
        ];
        if ($this->is17) {
            $hooks['displayProductAdditionalInfo'] = $this->l('Between social buttons and reassurance block');
            $hooks['displayReassurance'] = $this->l('After reassurance block');
        } else {
            $hooks['displayProductDeliveryTime'] = $this->l('Above social buttons');
        }

        $hooks['displayLeasing'] = $this->l('Custom hook at your chosen location (displayLeasing)');

        if (!$form_builder) {
            return $hooks;
        } else {
            $hooks_ordered = [];
            foreach ($hooks as $hook => $desc) {
                $hooks_ordered[] = [
                    'id' => $hook,
                    'name' => $desc,
                ];
            }

            return $hooks_ordered;
        }
    }

    public function getAvailableLogoSizes($form_builder = false) {
        $sizes = [
            'small' => $this->l('Small'),
            'medium' => $this->l('Medium')
        ];

        if (!$form_builder) {
            return $sizes;
        } else {
            $sizes_ordered = [];
            foreach ($sizes as $key => $size) {
                $sizes_ordered[] = [
                    'id' => $key,
                    'name' => $size
                ];
            }

            return $sizes_ordered;
        }
    }

    public function setRedirectAlert($message, $type, $isHtml = true)
    {
        if (is_array($message)) {
            $line = $isHtml ? '<br>' : PHP_EOL;
            $message = implode($line, $message);
        }
        $type = ($type == 'error') ? 'redirect_errors' : 'redirect_messages';
        $this->context->cookie->__set($type, $message);
    }

    /**
     * Returns error/success messages set before redirect
     *
     * @return string HTML
     */
    public function getRedirectAlerts($isHtml = true)
    {
        $html = '';

        if (isset($this->context->cookie->redirect_errors)) {
            $html .= $isHtml ? $this->displayError($this->context->cookie->redirect_errors) : $this->context->cookie->redirect_errors;
            unset($this->context->cookie->redirect_errors);
        }
        if (isset($this->context->cookie->redirect_messages)) {
            $html .= $isHtml ? $this->displayConfirmation($this->context->cookie->redirect_messages) : $this->context->cookie->redirect_messages;
            unset($this->context->cookie->redirect_messages);
        }

        return $html;
    }

    public function regenerateAllPaymentTemplates() {
        $options = InbankPayment::getAllPaymentMethods();
        if (!empty($options)) {
            foreach ($options as $option) {
                try {
                    $class_name = $option->class_name;
                    $product = new $class_name($option, $this);
                    $product->generateTemplate($class_name::INBANK_PAYMENT, true);
                } catch (Exception $e) {
                    InbankLogs::addLog(0, 'Failed to regenerate templates. '.$e->getMessage());
                }
            }
        }
    }

    /**
     * Updates and regenerates templates
     */
    public function updateAllProducts() {
        $options = InbankPayment::getAllPaymentMethods();
        if (!empty($options)) {
            /**
             * @var $option InbankPayment
             */
            foreach ($options as $option) {
                try {
                    $class_name = $option->class_name;
                    /**
                     * @var $product InbankBalticsProduct|AbstractPayment
                     */
                    $product = new $class_name($option, $this);
                    $product->updateSelf(true);
                } catch (Exception $e) {
                    InbankLogs::addLog(0, 'Failed to update products. '.$e->getMessage());
                }
            }
        }
    }

    /**
     * @param $field array array with $key as language id and $value as description text you want to show
     */
    public function getMultilangDesc($field, $prefix = "", $suffix = "", $skip_languages = []) {
        $this->context->smarty->assign([
            'multilang_desc' => $field,
            'context_lang' => $this->context->language->id,
            "inbank_prefix" => $prefix,
            "inbank_suffix" => $suffix,
            "inbank_skip_lang" => $skip_languages
        ]);
        $template = false;
        try {
            $template = $this->context->smarty->fetch(INBANK_MODULE_DIR.'views/templates/admin/multilang_desc.tpl');
        } catch (Exception $e) {}

        return $template;
    }

    public function generateHash($string) {
        if (method_exists(Tools::class, 'hash')) {
            return Tools::hash($string);
        } else if (method_exists(Tools::class, 'encrypt')) {
            return Tools::encrypt($string);
        }
    }


}
