<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class PhraseApp
{

    public static function buildTranslationFiles(
        $payment,
        $locale_id,
        $locale_token,
        $template,
        $force = false,
        $api_url = null
    ) {
        if ($api_url === null) {
            $api_url = 'https://api.phrase.com/v2/projects/';
        }
        if (empty($locale_id) || empty($locale_token)) {
            return false;
        }
        if (!is_dir(INBANK_MODULE_DIR.'cache')) {
            if (!mkdir(INBANK_MODULE_DIR.'cache')) {
                self::addLog(
                    0,
                    'Failed to create cache directory when trying to build translation templates.',
                    'warning',
                    '',
                    'PhraseApp.php:buildTranslationFiles'
                );

                return false;
            }
        }

        //phrase file dir
        $phrase_json = INBANK_MODULE_DIR.'cache/phrase_trans_'.$locale_id.'.json';
        $file_time = false;
        $now = time();
        if (file_exists($phrase_json)) {
            $file_time = filemtime($phrase_json);
        }
        //checking if file is older than 24 hours. Not accurate on DST, but we don't need that here.
        if ($force === true || $file_time == false || ($now - $file_time) > 86400) {
            //clearing the file
            file_put_contents($phrase_json, "");
            $api_url = $api_url.(Tools::substr($api_url, -1) == '/' ? $locale_id : '/'.$locale_id)
                .'/translations?per_page=100';
            //generating a new translation json
            $phrase_data = self::getTranslations($api_url, $locale_token, $phrase_json);
            if (!empty($phrase_data))
                file_put_contents($phrase_json, json_encode($phrase_data));
        }
        self::createTranslationFiles($phrase_json, $template, $payment);
    }

    private static function addLog(
        $id_order,
        $message,
        $type = 'warning',
        $product_code = "",
        $endpoint = "",
        $response = ""
    ) {
        InbankLogs::addLog($id_order, $message, $type, $product_code, $endpoint, $response);
    }

    private static function getTranslations($api_url, $locale_token, $directory, $data = [], $page = 1)
    {
        $curl = curl_init();
        $headers = [];
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Authorization: token '.$locale_token,
            'Content-Type: application/json',
        ]);
        curl_setopt($curl, CURLOPT_URL, $api_url.'&page='.$page);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_AUTOREFERER, 1);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt(
            $curl,
            CURLOPT_HEADERFUNCTION,
            function ($ch, $header) use (&$headers)
            {
                $len = Tools::strlen($header);
                $header = explode(':', $header, 2);
                // ignore invalid headers
                if (count($header) < 2) {
                    return $len;
                }

                $headers[Tools::strtolower(trim($header[0]))][] = trim($header[1]);

                return $len;
            }
        );


        $response = curl_exec($curl);
        $response_code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        if ($response_code === 200) {
            $data[] = json_decode($response);
            $next_page = $page;
            if (isset($headers["pagination"])) {
                $pagination = json_decode(reset($headers["pagination"]));
                $next_page = isset($pagination->next_page) && (int)$pagination->next_page > $page ? (int)$pagination->next_page : $page;
            }

            if ($page != $next_page)
                return self::getTranslations($api_url, $locale_token, $directory, $data, $next_page);
            else
                return $data;

        } else {
            self::addLog(
                0,
                'Failed to connect to API server. Response code: '.$response_code,
                'warning',
                '',
                'PhraseApp.php:getTranslations'
            );
            //todo log failed to connect to api server;
        }
    }

    private static function createTranslationFiles($translations_directory, $template_type, $payment_method)
    {
        if (!is_a($payment_method, 'InbankPayment')) {
            return false;
        }

        //loading translations
        $trans_file = json_decode(Tools::file_get_contents($translations_directory));
        if ($trans_file === null) {
            self::addLog(
                0,
                'Failed to parse translations. Try deleting '.$translations_directory.' to get new translations.',
                'warning',
                '',
                'PhraseApp:createTranslationFiles'
            );

            return false;
        }
        $grouped_translations = [];
        foreach ($trans_file as $page) {
            foreach ($page as $translation) {
                $grouped_translations[$translation->locale->name][$translation->key->name] = $translation->content;
            }
        }
        $languages = Language::getLanguages(false);

        try {
            $default_template = Tools::file_get_contents(INBANK_MODULE_DIR.'product_templates/'.$template_type.'.html');
        } catch (Exception $e) {
            self::addLog(0, 'Failed to get Inbank product template. '.$e->getMessage());

            return false;
        }
        $payment_method = new InbankPayment($payment_method->id);
        $class = $payment_method->class_name;
        $template = null;
        try {
            $template = $class::getPhraseTemplateVars(
                $template_type,
                ['default_period' => $payment_method->default_period]
            );
        } catch (Exception $e) {
        }
        if ($template !== null) {
            //template is present
            foreach ($languages as $lang) {
                $generated_template = [];
                $iso = $lang['iso_code'];
                $iso_translated = self::localeTranslator($iso);
                $id_lang = $lang['id_lang'];
                //checking if this product gets it's title from phrase
                $query = [];

                if (!isset($grouped_translations[$iso_translated]))
                    continue;

                if(isset($template['local']) && is_array($template['local']) && !empty($template['local'])) {
                    foreach ($template['local'] as $key => $trans_name) {
                        $payment_method->$key[$id_lang] = '';
                        if(isset($grouped_translations[$iso_translated][$trans_name])) {
                            $query[$key] = $grouped_translations[$iso_translated][$trans_name];
                            $payment_method->$key[$id_lang] = $grouped_translations[$iso_translated][$trans_name];
                        }
                    }
                }

                if (isset($template['title'])) {
                    $title = $template['title'];
                    if (isset($grouped_translations[$iso_translated][$title])) {
                        $title_translated = $grouped_translations[$iso_translated][$title];
                        $query['title'] = $title_translated;
                        if (!is_array($payment_method) || $payment_method->custom_title[$id_lang] === $payment_method->title[$id_lang]) {
                            $query['custom_title'] = $title_translated;
                            $payment_method->custom_title[$id_lang] = $title_translated;
                        }
                        $payment_method->title[$id_lang] = $title_translated;
                    }
                }
                if (isset($template['granted'])) {
                    $granted = $template['granted'];
                    if (isset($grouped_translations[$iso_translated][$granted])) {
                        $granted_translated = $grouped_translations[$iso_translated][$granted];
                        $query['granted'] = $granted_translated;
                        $payment_method->granted[$id_lang] = $granted_translated;
                    }
                }
                if (isset($template['additional_details'])) {
                    $details = $template['additional_details'];
                    if (isset($grouped_translations[$iso_translated][$details])) {
                        $details_translated = $grouped_translations[$iso_translated][$details];
                        $query['additional_details'] = $details_translated;
                        if (
                            isset($payment_method->custom_additional_details[$id_lang]) &&
                            isset($payment_method->additional_details[$id_lang]) &&
                            $payment_method->custom_additional_details[$id_lang] === $payment_method->additional_details[$id_lang]
                        )
                        {
                            $query['custom_additional_details'] = $details_translated;
                            $payment_method->custom_additional_details[$id_lang] = $details_translated;
                        }

                        $payment_method->additional_details[$id_lang] = $details_translated;
                    }
                }
                if (!empty($query)) {
                    Db::getInstance()->update(
                        'inbank_payment_lang',
                        $query,
                        ' id_payment = '.$payment_method->id.' AND id_lang = '.$id_lang
                    );
                }
                //regenerating product template
                if (isset($template['body'])) {
                    $body = $template['body'];
                    if (is_array($body)) {
                        foreach ($body as $placeholder => $tem_var) {
                            if (isset($grouped_translations[$iso_translated][$tem_var])) {
                                $generated_template[$placeholder] = $grouped_translations[$iso_translated][$tem_var];
                            } else {
                                $generated_template[$placeholder] = "";
                            }
                        }
                    }
                }
                if (!empty($generated_template)) {
                    file_put_contents(
                        INBANK_MODULE_DIR.'cache/'.$payment_method->id."_".(int)$payment_method->debug_mode."_".$template_type."_".$id_lang.".html",
                        self::replacePlaceholders($default_template, $generated_template)
                    );
                }
            }
        }
    }

    /**
     * @param $payment_object InbankPayment
     * @param $localization_id string payment localization id
     * @param $localization_token string payment localization token
     * @return bool;
     */
    public static function fillSinglePaymentTranslations($payment_object)
    {
        if (!isset($payment_object->class_name)) {
            return false;
        }

        $class = $payment_object->class_name;

        if (!method_exists($class, 'getPhraseTemplateVars')) {
            return false;
        }

        $api_url = 'https://api.phrase.com/v2/projects/'.$payment_object->localization_id.'/translations?per_page=100';
        $phrase_json = INBANK_MODULE_DIR.'cache/phrase_trans_'.$payment_object->localization_id.'.json';
        if (!file_exists($phrase_json) || time() - filemtime($phrase_json) > 3600 * 24) {
            self::getTranslations($api_url, $payment_object->localization_token, $phrase_json);
        }

        try {
            $translations = json_decode(Tools::file_get_contents($phrase_json));
        } catch (Exception $e) {
            self::addLog(0, 'Failed to load translations');

            return false;
        }

        $grouped_translations = [];
        foreach ($translations as $page) {
            foreach ($page as $translation) {
                $grouped_translations[$translation->locale->name][$translation->key->name] = $translation->content;
            }
        }

        $languages = Language::getLanguages(false);

        $vars = $class::getPhraseTemplateVars('', ['default_period' => $payment_object->default_period]);

        if (empty($vars)) {
            return false;
        }

        foreach ($languages as $lang) {
            $iso = $lang['iso_code'];
            $iso_translated = self::localeTranslator($iso);
            $id_lang = $lang['id_lang'];

            if (isset($vars['title'])) {
                $title = $vars['title'];
                if (isset($grouped_translations[$iso_translated][$title])) {
                    $payment_object->title[$id_lang] = $grouped_translations[$iso_translated][$title];
                }
            }

            if (isset($vars['additional_details'])) {
                $additional_details = $vars['additional_details'];
                if (isset($grouped_translations[$iso_translated][$additional_details])) {
                    $payment_object->additional_details[$id_lang] = $grouped_translations[$iso_translated][$additional_details];
                }
            }
        }

        return true;
    }

    public static function replacePlaceholders($template, $placeholders)
    {
        $keys = [];
        $values = [];
        foreach ($placeholders as $key => $value) {
            $keys[] = "{".$key."}";
            $values[] = $value;
        }

        return str_replace($keys, $values, $template);
    }

    public static function localeTranslator($iso_code)
    {
        switch ($iso_code) {
            case 'et':
            case 'ee':
                return 'et-EE';
            case 'lv':
                return 'lv-LV';
            case 'pl':
                return 'pl-PL';
            case 'ru':
                return 'ru';
            case 'cs':
            case 'cz':
                return 'cs';
            case 'en':
            case 'us':
            default:
                return 'en';
        }
    }
}
