<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

abstract class InbankBalticsProduct extends AbstractPayment
{

    public $object;
    public $module;
    public $api;
    private $uses_granted = 1;
    public $product_code;

    /**
     * InbankBalticsProduct constructor.
     * @param $object_model InbankPayment
     * @param $module Inbank
     * @throws PrestaShopException
     * @throws Exception
     */
    public function __construct($object_model, $module)
    {
        if (is_array($object_model)) {
            $object_model = ObjectModel::hydrateCollection("InbankPayment", $object_model);
        }
        if (is_a($object_model, "InbankPayment")) {
            $this->object = $object_model;
        } else {
            throw new Exception('Object must be an instance of InbankPayment');
        }
        $this->module = $module;
        $this->product_code = $this->getProductCode();
        $this->api = new InbankBalticsAPI($this->object, $this->getApiUrl());
    }

    /**
     * Returns Product settings form suitable for prestashop formbuilder
     * @return array[]
     */
    public function productApiForm()
    {
        $class_name = $this->object->class_name;
        $large_calc = array(
            'type' => 'switch',
            'label' => $this->module->l('Enable large calculator'),
            'desc' => $this->module->l('Shows large calculator in product page'),
            'name' => 'calc_large',
            'is_bool' => true,
            'values' => array(
                array(
                    'id' => 'active_on',
                    'value' => true,
                    'label' => $this->module->l('Enable'),
                ),
                array(
                    'id' => 'active_off',
                    'value' => false,
                    'label' => $this->module->l('Disable'),
                ),
            ),
        );

        $file_name = $this->object->id.'_'.$this->object->logo_size.'.jpg';
        $thumb_path = INBANK_IMG_DIR.$file_name;
        $thumb_url = ImageManager::thumbnail(
            $thumb_path,
            'inbank_custom_logo_'.(int) $this->object->id . '.jpg',
            200,
            'jpg',
            true,
            true
        );
        $thumb_size = file_exists($thumb_path) ? filesize($thumb_path) / 1000 : false;
        $delete_url = 'delete_url_disabled';

        if (Tools::getIsset('updateinbank_payment')) {
            $delete_url = 'delete_url';
            $custom_logo_delete_url = Context::getContext()->link->getAdminLink('AdminModules', true) . '&configure=inbank&tab_module=payments_gateways&module_name=inbank&updateinbank_payment=1&id_payment=' . $this->object->id.'&deletecustomlogo=1';
        }
        else
            $custom_logo_delete_url = '';

        $form = array(
            'form' => array(
                'legend' => array(
                    'title' => method_exists($this->object->class_name,
                        'getDefaultTitle') ? $class_name::getDefaultTitle() : $this->object->class_name,
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'hidden',
                        'name' => 'id_payment',
                    ),
                    array(
                        'type' => 'hidden',
                        'name' => 'class_name',
                    ),
                    array(
                        'type' => 'hidden',
                        'name' => 'id_country',
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->module->l('Product enabled'),
                        'name' => 'enabled',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->module->l('Enable'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->module->l('Disable'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->module->l('Enable test mode'),
                        'name' => 'debug_mode',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->module->l('Enable'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->module->l('Disable'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Live product code'),
                        'name' => 'product_code',
                        'class' => 'inbank_input_production',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Test product code'),
                        'name' => 'product_code_debug',
                        'class' => 'inbank_input_debug',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Live API key'),
                        'desc' => $this->module->l('Live API key provided by Inbank.'),
                        'name' => 'api_key',
                        'class' => 'inbank_input_production',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Test API key'),
                        'desc' => $this->module->l('Test API key provided by Inbank.'),
                        'name' => 'api_key_debug',
                        'class' => 'inbank_input_debug',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Live shop UUID'),
                        'desc' => $this->module->l('Live shop unique identifier provided by Inbank.'),
                        'name' => 'shop_uuid',
                        'class' => 'inbank_input_production',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Test shop UUID'),
                        'desc' => $this->module->l('Test shop unique identifier provided by Inbank.'),
                        'name' => 'shop_uuid_debug',
                        'class' => 'inbank_input_debug',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Localization ID'),
                        'desc' => $this->module->l('Localization ID provided by Inbank.'),
                        'name' => 'localization_id',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Localization token'),
                        'desc' => $this->module->l('Localization token provided by Inbank.'),
                        'name' => 'localization_token',
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->module->l('Display logo'),
                        'desc' => $this->module->l('Whether the logo appears next to the payment method at checkout.'),
                        'name' => 'use_logo',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->module->l('Enable'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->module->l('Disable'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->module->l('Logo size'),
                        'name' => 'logo_size',
                        'options' => [
                            'query' => $this->module->getAvailableLogoSizes(true),
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ),
                    array(
                        'type' => 'file',
                        'label' => $this->module->l('Custom logo'),
                        'name' => 'custom_logo',
                        'display_image' => true,
                        'show_thumbnail' => true,
                        'image' => $thumb_size > 0 && $this->object->id ? $thumb_url : false,
                        $delete_url => $custom_logo_delete_url
                    ),
                    array(
                        'type' => $this->hasCalcSmall() ? 'switch' : 'hidden',
                        'label' => $this->module->l('Enable calculations'),
                        'desc' => $this->module->l('Whether calculated monthly amounts should be presented on product pages.'),
                        'name' => 'calc_small',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->module->l('Enable'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->module->l('Disable'),
                            ),
                        ),
                    ),
                    array(
                        'type' => $this->hasCalcSmall() ? 'select' : 'hidden',
                        'options' => [
                            'query' => $this->module->getAvailablePositions(true),
                            'id' => 'id',
                            'name' => 'name',
                        ],
                        'label' => $this->module->l('Calculation position'),
                        'name' => 'calc_small_position',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Title'),
                        'lang' => true,
                        'name' => 'custom_title',
                        'placeholder' => '',
                    ),
                    /*
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Title'),
                        'lang' => true,
                        'name' => 'title',
                        'disabled' => true,
                        'readonly' => true,
                    ),*/
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Minimum order amount'),
                        'name' => 'minimum_order_amount',
                        'readonly' => true,
                        'disabled' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Maximum order amount'),
                        'name' => 'maximum_order_amount',
                        'readonly' => true,
                        'disabled' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Period'),
                        'name' => 'period',
                        'readonly' => true,
                        'disabled' => true,
                    ),
                    array(
                        'type' => 'hidden',
                        'label' => $this->module->l('Default payment period'),
                        'name' => 'default_period',
                        'readonly' => true,
                        'disabled' => true,
                    ),
                    array(
                        'type' => 'hidden',
                        'label' => $this->module->l('Needs merchant approval'),
                        'name' => 'needs_approval',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->module->l('Yes'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->module->l('No'),
                            ),
                        ),
                        'readonly' => true,
                        'disabled' => true,
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => $this->module->l('Additional details'),
                        'name' => 'custom_additional_details',
                        'lang' => true,
                    ),
                ),
                'submit' => array(
                    'title' => $this->module->l('Save'),
                ),
                'buttons' => array(
                    array(
                        'href' => Context::getContext()->link->getAdminLink('AdminModules').'&configure='.$this->module->name.'&tab_module'.$this->module->tab.'&module_name='.$this->module->name,
                        'title' => $this->module->l('Back to list'),
                        'icon' => 'process-icon-back',
                    ),
                ),
            ),
        );

        return $form;
    }

    /**
     * Generates template
     * @param $template
     * @param  false  $force
     */
    public function generateTemplate($template, $force = false)
    {
        PhraseApp::buildTranslationFiles(
            $this->object,
            $this->object->localization_id,
            $this->object->localization_token,
            $template,
            $force
        );
    }

    /**
     * Handle customer payments
     * @param $id_order int Order object id
     */
    public function handlePayment($id_order)
    {
        $order = new Order($id_order);
        $customer = new Customer($order->id_customer);
        $address = new Address($order->id_address_delivery);
        $currency = new Currency($order->id_currency);
        $context = Context::getContext();
        /*
        $redirect_back_url = $context->link->getPageLink('order-confirmation', null, $context->language->id, [
            'id_cart' => $order->id_cart,
            'id_module' => $this->module->id,
            'id_order' => $order->id,
            'key' => $order->secure_key,
        ]);
        */

        $endpoint = "pos_sessions";

        $request_body = [];
        //general info
        $request_body['product_code'] = $this->getProductCode();
        $request_body['total_amount'] = $order->total_paid_tax_incl;
        $request_body['currency'] = $currency->iso_code;
        $request_body['locale'] = $context->language->language_code;
        //customer data
        $request_body['customer_data']['first_name'] = $customer->firstname;
        $request_body['customer_data']['last_name'] = $customer->lastname;
        $request_body['customer_contact_data']['email'] = $customer->email;
        if (!empty($address->phone_mobile)) {
            $request_body['customer_contact_data']['mobile'] = $address->phone_mobile;
        } else {
            if (!empty($address->phone)) {
                $request_body['customer_contact_data']['mobile'] = $address->phone;
            }
        }
        //order info
        $request_body['purchase']['purchase_reference'] = $order->reference;
        //merchant info
        $request_body['purchase']['merchant']['merchant_name'] = $context->shop->name;
        $request_body['purchase']['merchant']['merchant_domain_name'] = (Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://').htmlspecialchars($_SERVER['HTTP_HOST'],
                ENT_COMPAT, 'UTF-8').__PS_BASE_URI__;
        $request_body['purchase']['merchant']['merchant_order_reference'] = $order->id;
        //return urls
        $request_body['partner_urls']['return_url'] = $context->link->getModuleLink($this->module->name,
            'return');//$redirect_back_url;
        $request_body['partner_urls']['cancel_url'] = $context->link->getModuleLink($this->module->name, 'cancel');
        $request_body['partner_urls']['callback_url'] = $context->link->getModuleLink($this->module->name, 'callback');

        $request_body['integration_info']['method'] = 'plugin';
        $request_body['integration_info']['platform'] = 'prestashop';
        $request_body['integration_info']['version'] = $this->module->version;

        $response = $this->api->initiateSession($request_body);
        if (isset($response->uuid)) {
            $details = new InbankDetail();
            $details->id_order = $order->id;
            $details->id_cart = $order->id_cart;
            $details->id_payment = $this->object->id;
            $details->session_uuid = $response->uuid;
            try {
                $details->save();
                Tools::redirectLink($response->redirect_url);
            } catch (Exception $e) {
                self::addLog(
                    $order->id,
                    'Failed to save server response. '.$e->getMessage(),
                    'error',
                    $this->getProductCode(),
                    $endpoint,
                    json_encode($response)
                );
            }
        } else {
            self::addLog(
                $order->id,
                'Bad response from server.',
                'error',
                $this->getProductCode(),
                $endpoint,
                json_encode($response)
            );
            Tools::redirect($this->context->link->getPageLink('cart'));
        }
    }


    public function getCalcLargeTemplate($product_price)
    {
        if (!$this->has_large_calc) {
            return false;
        }
        $shop_uuid = $this->object->debug_mode ? $this->object->shop_uuid_debug : $this->object->shop_uuid;
        $calc_link = $this->getEposUrl();
        $calc_link = Tools::substr($calc_link, -1) == '/' ? $calc_link : $calc_link.'/';
        $calc_link .= 'calculator/indivy/plan?product_code='
            .$this->getProductCode()
            .'&lang='.PhraseApp::localeTranslator(Context::getContext()->language->iso_code)
            .'&amount='.$product_price
            .'&shop_uuid='.$shop_uuid;

        $placeholders = [
            'calc_link' => $calc_link,
        ];
        $path = INBANK_MODULE_DIR.'product_templates/'.self::INBANK_CALC_LARGE.'.html';
        if ($template = Tools::file_get_contents($path)) {
            return self::replacePlaceholders($template, $placeholders);
        }

        return false;
    }

    public function getCurrencySymbolByCode($code)
    {
        $code = Tools::strtoupper($code);
        $currency = Currency::getIdByIsoCode($code);
        $currency = new Currency($currency);
        if (isset($currency->symbol)) {
            return $currency->symbol;
        } else {
            if (isset($currency->sign)) {
                return $currency->sign;
            } else {
                return $code;
            }
        }
    }

    /**
     * Checks if any single product needs to be updated and updates it
     * @param $module Inbank
     * @throws PrestaShopException
     */
    public static function updatePaymentData($module)
    {
        //taking one payment
        $yesterday = date("Y-m-d", strtotime("yesterday"));
        $sql = 'SELECT id_payment FROM '._DB_PREFIX_.'inbank_payment WHERE api_last_upd < "'.$yesterday.'" OR api_last_upd IS NULL';
        $payment = (int)Db::getInstance()->getValue($sql);
        if ($payment) {
            $payment_obj = new InbankPayment($payment);

            $class_name = $payment_obj->class_name;

            /**
             * @var $inbank_product AbstractPayment|InbankBalticsProduct
             */
            $inbank_product = new $class_name($payment_obj, $module);
            $inbank_product->updateSelf(true);
        }
    }

    /**
     * fills object with data from API. If regenerate templates set to true, saves the object before regenerating
     * @param $regenerate_templates bool should we regenerate templates. If set to true, saves the object before
     *     regenerating
     * @throws PrestaShopException
     */
    public function updateSelf($regenerate_templates = false, $product_info = [])
    {
        if (empty($product_info)) {
            $product_info = $this->api->getInfo();
        }
        if (isset($product_info->code) && $product_info->code === $this->getProductCode()) {
            foreach (Language::getLanguages(false) as $lang) {
                if (isset($this->object->title[$lang['id_lang']]) && $this->object->title[$lang['id_lang']] === $this->object->custom_title[$lang['id_lang']]) {
                    $this->object->custom_title[$lang['id_lang']] = $product_info->name;
                }
                $this->object->title[$lang['id_lang']] = $product_info->name;
            }
            $this->object->minimum_order_amount = $product_info->parameters->min_amount;
            $this->object->maximum_order_amount = $product_info->parameters->max_amount;
            $this->object->period = implode(",", $product_info->parameters->period_options);
            $this->object->default_period = $product_info->parameters->default_period;
            $this->object->max_period = !empty($product_info->parameters->period_options) ? max($product_info->parameters->period_options) : $product_info->parameters->default_period;
            $this->object->default_payment_day = !empty($product_info->parameters->default_payment_day) ? $product_info->parameters->default_payment_day : 10;
            $this->object->needs_approval = isset($product_info->parameters->contract_activation_requires_partner_approval) ? $product_info->parameters->contract_activation_requires_partner_approval : false;
            $this->object->down_payment = isset($product_info->parameters->down_payment_enabled) ? $product_info->parameters->down_payment_enabled : false;
            $this->object->api_last_upd = date('Y-m-d H:i:s');
        }
        if ($regenerate_templates) {
            $this->object->save();
            $this->generateTemplate(self::INBANK_PAYMENT);
            if ($this->object->calc_small) {
                $this->generateTemplate(self::INBANK_CALC_SMALL);
            }
            if ($this->object->down_payment) {
                $this->generateTemplate(self::INBANK_CALC_SMALL_DOWNPAYMENT);
                $this->generateTemplate(self::INBANK_DOWNPAYMENT);
            }
        }
    }

    public function isAcceptedCurrency($accepted_currency, $supplied_currency)
    {
        $supplied_currency_code = $supplied_currency;
        if (is_numeric($supplied_currency) && $this->module->is17) {
            $supplied_currency_code = Currency::getIsoCodeById($supplied_currency);
        }

        return $accepted_currency === $supplied_currency_code;
    }

    public function hasCalcSmall() {
        return true;
    }

    abstract public static function getSubtype();
}