<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankPayment extends ObjectModel
{

    /** @var int object identifier */
    public $id_payment;

    /** @var int shop identifier */
    public $id_shop;

    /** @var string class identifier */
    public $class_name;

    /** @var string Inbank product identifier */
    public $product_code;

    /** @var string Inbank product identifier */
    public $product_code_debug;

    /** @var int country identifier */
    public $id_country;

    /** @var bool wether payment is enabled or disabled */
    public $enabled = 0;

    /** @var bool whether debug mode is on */
    public $debug_mode;

    /** @var bool show small calculator */
    public $calc_small;

    /** @var bool show big calculator */
    public $calc_large;

    /** @var string small calculator hook */
    public $calc_small_position;

    /** @var string LIVE api key */
    public $api_key;

    /** @var string DEBUG api key */
    public $api_key_debug;

    /** @var string LIVE api token */
    public $shop_uuid;

    /** @var string DEBUG api token */
    public $shop_uuid_debug;

    /** @var string Localization id */
    public $localization_id;

    /** @var string Localization token */
    public $localization_token;

    /** @var string Title of the payment option */
    public $title;

    /** @var string custom payment option title */
    public $custom_title;

    /** @var string Wether to show logo in the payment options or not */
    public $use_logo = 1;

    /** @var string Logo size used in payment window */
    public $logo_size = 'small';

    /** @var float Minimum order amount for the payment option to show up */
    public $minimum_order_amount;

    /** @var float Maximum order amount for the payment option to show up */
    public $maximum_order_amount;

    /** @var string available periods */
    public $period;

    /** @var int default period */
    public $default_period;

    /** @var int max period */
    public $max_period;

    /** @var int payment date */
    public $default_payment_day;

    /** @var string additional details shown near payment option */
    public $additional_details;

    /** @var string custom additional details */
    public $custom_additional_details;

    /** @var bool does this payment method needs approval from merchant */
    public $needs_approval = 0;

    /** @var bool determines if we need to show the down payment */
    public $down_payment = 0;

    /** @var string granted text */
    public $granted;

    /** @var string */
    public $representative_example;

    /** @var string */
    public $euribor_1m;

    /** @var string */
    public $euribor_3m;

    /** @var string */
    public $euribor_6m;

    /** @var string */
    public $euribor_12m;

    /** @var int */
    public $restriction_type;

    /** @var datetime date time of last update */
    public $api_last_upd = null;

    public static $definition = array(
        'table' => 'inbank_payment',
        'primary' => 'id_payment',
        'multilang' => true,
        'fields' => array(
            'id_shop' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'class_name' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'product_code' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'product_code_debug' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'id_country' => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'enabled' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'debug_mode' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'calc_small' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'calc_large' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'calc_small_position' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'api_key' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'api_key_debug' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'shop_uuid' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'shop_uuid_debug' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'localization_id' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'localization_token' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'title' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'custom_title' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'use_logo' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'logo_size' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'minimum_order_amount' => array('type' => self::TYPE_FLOAT, 'validate' => 'isFloat', 'required' => false),
            'maximum_order_amount' => array('type' => self::TYPE_FLOAT, 'validate' => 'isFloat', 'required' => false),
            'period' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'default_period' => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => false),
            'max_period' => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => false),
            'default_payment_day' => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => false),
            'additional_details' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'custom_additional_details' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'needs_approval' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'down_payment' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'granted' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'representative_example' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'euribor_1m' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'euribor_3m' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'euribor_6m' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'euribor_12m' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'restriction_type' => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => false),
            'api_last_upd' => array('type' => self::TYPE_DATE, 'validate' => 'isDate', 'required' => false, 'allow_null' => true),
        ),
    );

    public static function checkForExistanceByClassName($class_name)
    {
        return (bool)Db::getInstance()->getValue("SELECT class_name FROM " . _DB_PREFIX_ . "inbank_payment 
                                                WHERE class_name = '" . pSQL($class_name) . "'");
    }

    public static function checkForExistanceByProductCode($product_code)
    {
        return (bool)Db::getInstance()->getValue("SELECT product_code FROM " . _DB_PREFIX_ . "inbank_payment 
                                                WHERE product_code = '" . pSQL($product_code) . "'");
    }

    public function toArray()
    {
        $obj_to_array = [];
        $obj_to_array[self::$definition["primary"]] = $this->id;
        foreach (self::$definition["fields"] as $field => $type) {
            if (is_array($this->$field)) {
                foreach ($this->$field as $lang_id => $value) {
                    $obj_to_array[$field][$lang_id] = $value;
                }
            } else {
                $obj_to_array[$field] = $this->$field;
            }
        }

        return $obj_to_array;
    }

    /**
     * Gets all payment methods within the supplied price
     * @param $product_price float product price
     */
    public static function getEligiblePaymentMethods(
        $product_price,
        $id_country = null,
        $lang = null,
        $hook = null,
        $calc_small = null,
        $calc_large = null,
        $id_product = null
    ) {
        if ($lang === null) {
            $lang = (int)Context::getContext()->language->id;
        } else {
            $lang = (int)$lang;
        }

        if ($id_country == null)
            $id_country = Configuration::get('PS_COUNTRY_DEFAULT');

        $product_price = (float)$product_price;
        $sql = 'SELECT ip.*, ipl.* FROM ' . _DB_PREFIX_ . 'inbank_payment ip 
                LEFT JOIN ' . _DB_PREFIX_ . 'inbank_payment_lang ipl ON ip.id_payment = ipl.id_payment
                AND ipl.id_lang = ' . $lang . '
                WHERE ip.enabled = 1 AND ' . $product_price . ' >= ip.minimum_order_amount
                AND ' . $product_price . ' <= ip.maximum_order_amount';
        if ($id_country !== null) {
            $sql .= " AND id_country = " . (int)$id_country;
        }
        if ($hook !== null) {
            if ($hook === 'displayProductPriceBlock') {
                $sql .= ' AND (calc_small_position = "' . pSQL($hook) . '" OR calc_small_position = "")';
            } else {
                $sql .= ' AND calc_small_position = "' . pSQL($hook) . '"';
            }
        }
        if ($calc_small !== null) {
            $sql .= ' AND calc_small = ' . (int)$calc_small;
        }
        if ($calc_large !== null) {
            $sql .= ' AND calc_large = ' . (int)$calc_large;
        }

        $payments = Db::getInstance()->executeS($sql);

        if($id_product && !empty($payments)) {
            foreach ($payments as $key => $payment) {
                if(!self::checkRestrictions($id_product, $payment['id_payment'], $payment['restriction_type'])) {
                    unset($payments[$key]);
                }
            }
        }

        //reset array keys
        $payments = array_values($payments);
        return self::hydrateCollection(self::class, $payments, $lang);
    }

    public static function checkRestrictions($id_product, $id_payment, $restriction_type)
    {
        $payment_category_restrictions = Db::getInstance()->executeS(
            'SELECT id_category FROM '._DB_PREFIX_.'inbank_category_restriction 
                        WHERE id_payment = '.(int)$id_payment
        );
        $payment_category_restrictions = array_column($payment_category_restrictions, 'id_category');

        $payment_product_restrictions = Db::getInstance()->executeS(
            'SELECT id_product FROM '._DB_PREFIX_.'inbank_product_restriction 
                        WHERE id_payment = '.(int)$id_payment
        );
        $payment_product_restrictions = array_column($payment_product_restrictions, 'id_product');

        if(empty($payment_category_restrictions) && empty($payment_product_restrictions)) {
            return true;
        }

        if(!is_array($id_product)) {
            $id_product = [$id_product];
        }

        $category_exists = false;
        if (!empty($payment_category_restrictions)) {
            foreach ($id_product as $id) {
                $product_categories = Product::getProductCategories($id);
                $category_exists = false;
                foreach ($product_categories as $product_category) {
                    if (in_array($product_category, $payment_category_restrictions)) {
                        //checking if category exists
                        $category_exists = true;
                        break;
                    }
                }
                //we're checking if restriction type is AND, if it is we're checking for category existence
                if ($restriction_type == 0 && !$category_exists) {
                    return false;
                }
            }
        }

        if($restriction_type == 1 && $category_exists) {
            return true;
        }

        if(!empty($payment_product_restrictions)) {
            //checking if all products exist in payment restrictions
            return count($id_product) === count(array_intersect($id_product, $payment_product_restrictions));
        }

        return true;
    }

    /**
     * @return array of InbankPayment objects
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function getActivePaymentMethods()
    {
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'inbank_payment WHERE enabled = 1';
        $payments = Db::getInstance()->executeS($sql);

        return self::hydrateCollection(self::class, $payments);
    }

    public static function getAllPaymentMethods()
    {
        $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'inbank_payment ip
                LEFT JOIN ' . _DB_PREFIX_ . 'inbank_payment_lang ipl ON ip.id_payment = ipl.id_payment';
        $payments = Db::getInstance()->executeS($sql);

        return self::hydrateCollection(self::class, $payments);
    }

    /**
     * Add product restrictions to this payment method
     * @param $product_ids array
     * @return bool
     * @throws PrestaShopDatabaseException
     */
    public function addProductRestrictions($product_ids)
    {
        if(empty($product_ids)) {
            return true;
        }

        $res = true;

        foreach ($product_ids as $product_id) {
            $res &= Db::getInstance()->insert(
                'inbank_product_restriction',
                [
                    'id_payment' => $this->id,
                    'id_product' => (int)$product_id
                ],
                false,
                false,
                Db::INSERT_IGNORE
            );
        }

        return $res;
    }

    /**
     * Removes supplied product restrictions
     * @param $product_ids array product ids
     * @return bool
     */
    public function removeProductRestrictions($product_ids)
    {
        if(empty($product_ids)) {
            return true;
        }

        return Db::getInstance()->delete(
            'inbank_product_restriction',
            'id_product IN('.implode(',', $product_ids).') AND id_payment = '.$this->id
        );
    }


    /**
     * Deletes product restrictions which are not in supplied array and adds missing ones
     * @param $product_ids array product ids
     * @return bool
     * @throws PrestaShopDatabaseException
     */
    public function syncProductRestrictions($product_ids)
    {
        $current_restrictions = Db::getInstance()->executeS(
            'SELECT id_product FROM '._DB_PREFIX_.'inbank_product_restriction WHERE id_payment = '.$this->id
        );

        $current_restrictions = array_column($current_restrictions, 'id_product');
        $products_to_delete = array_diff($current_restrictions, $product_ids);
        $products_to_add = array_diff($product_ids, $current_restrictions);

        return $this->removeProductRestrictions($products_to_delete) && $this->addProductRestrictions($products_to_add);
    }

    /**
     * Returns array of product ids
     * @return array
     * @throws PrestaShopDatabaseException
     */
    public function getProductRestrictions()
    {
        return array_column(
            Db::getInstance()->executeS(
                'SELECT id_product FROM '._DB_PREFIX_.'inbank_product_restriction WHERE id_payment = '.$this->id
            ),
            'id_product'
        );
    }

    /**
     * Add category restrictions to this payment method
     * @param $category_ids array
     * @return bool
     * @throws PrestaShopDatabaseException
     */
    public function addCategoryRestrictions($category_ids)
    {
        if(empty($category_ids)) {
            return true;
        }

        $res = true;

        foreach ($category_ids as $category_id) {
            $res &= Db::getInstance()->insert(
                'inbank_category_restriction',
                [
                    'id_payment' => $this->id,
                    'id_category' => (int)$category_id
                ],
                false,
                false,
                Db::INSERT_IGNORE
            );
        }

        return $res;
    }

    /**
     * Removes supplied category restrictions
     * @param $category_ids array category ids
     * @return bool
     */
    public function removeCategoryRestrictions($category_ids)
    {
        if(empty($category_ids)) {
            return true;
        }

        return Db::getInstance()->delete(
            'inbank_category_restriction',
            'id_category IN('.implode(',', $category_ids).') AND id_payment = '.$this->id
        );
    }

    /**
     * Deletes category restrictions which are not in supplied array and adds missing ones
     * @param $category_ids array product ids
     * @return bool
     * @throws PrestaShopDatabaseException
     */
    public function syncCategoryRestrictions($category_ids)
    {
        $current_restrictions = Db::getInstance()->executeS(
            'SELECT id_category FROM '._DB_PREFIX_.'inbank_category_restriction WHERE id_payment = '.$this->id
        );

        $current_restrictions = array_column($current_restrictions, 'id_category');
        $categories_to_delete = array_diff($current_restrictions, $category_ids);
        $categories_to_add = array_diff($category_ids, $current_restrictions);

        return $this->removeCategoryRestrictions($categories_to_delete) && $this->addCategoryRestrictions($categories_to_add);
    }

    /**
     * Returns array of category ids
     * @return array
     * @throws PrestaShopDatabaseException
     */
    public function getCategoryRestrictions()
    {
        return array_column(
            Db::getInstance()->executeS(
                'SELECT id_category FROM '._DB_PREFIX_.'inbank_category_restriction WHERE id_payment = '.$this->id
            ),
            'id_category'
        );
    }
}
