<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankPaymentForm
{

    private $module;
    private $context;
    private $input_data = [
        'id_country' => '',
        'class_name' => '',
        'new_payment_action' => 'new_payment',
        'product_code' => '',
        'product_code_debug' => '',
        'debug_mode' => '',
        'api_key' => '',
        'api_key_debug' => '',
        'shop_uuid' => '',
        'shop_uuid_debug' => '',
        'localization_id' => '',
        'localization_token' => '',
    ];

    public function __construct($module)
    {
        $this->module = $module;
        $this->context = Context::getContext();
    }

    public function newProductForm()
    {
        $page = Tools::getValue('new_payment_action');
        if (Tools::isSubmit('class_name')) {
            $class_name = Tools::getValue('class_name');
            if ($class_name == 0) {
                $page = 'new_payment';
                $this->input_data['id_country'] = Tools::getValue('id_country');
            }
        }
        switch ($page) {
            case 'new_payment':
                return $this->loadSelectedPaymentForm();
            case 'get_available_payments':
                $inbankFormHelper = new InbankPaymentFormHelper($this->module);
                die($inbankFormHelper->filterPayments());
            default:
                $form = $this->newProductStep1();
                if (isset($form['form'])) {
                    return $this->buildPaymentForm($form, $this->input_data, 'newInbankPayment');
                } else {
                    return $form;
                }
        }
    }

    private function newProductStep1()
    {
        $this->context->smarty->assign(['step' => 1]);
        $formHelper = new InbankPaymentFormHelper($this->module);
        $countries = $formHelper->getAllProductCountries();
        $id_country = $this->input_data['id_country'] ? $this->input_data['id_country'] : reset($countries)['id_country'];
        $products = $formHelper->getPaymentsByCountry($id_country);

        $api_data = 'SELECT 
                            MAX(api_key) AS api_key, MAX(api_key_debug) AS api_key_debug, MAX(shop_uuid) AS shop_uuid,
                            MAX(shop_uuid_debug) AS shop_uuid_debug
                        FROM '._DB_PREFIX_.'inbank_payment
                        WHERE id_country = '.(int)$id_country;
        $api_data2 = 'SELECT 
                            MAX(localization_id) AS localization_id, MAX(localization_token) AS localization_token
                        FROM '._DB_PREFIX_.'inbank_payment';
        $api_data = Db::getInstance()->getRow($api_data);
        $api_data2 = Db::getInstance()->getRow($api_data2);
        $api_data = array_merge($api_data, $api_data2);

        foreach ($api_data as $key => $value) {
            if (isset($this->input_data[$key]) && empty($this->input_data[$key])) {
                $this->input_data[$key] = $value;
            }
        }

        if (empty($countries)) {
            return $this->module->displayWarning($this->module->l('There was an error trying to get Inbank Products. Please contact the Inbank team.'));
        }

        $form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->module->l('STEP 1: SELECT COUNTRY AND PRODUCT'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'hidden',
                        'name' => 'new_payment_action',
                        'value' => 'new_payment',
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->module->l('Country'),
                        'desc' => $this->module->l('Select country where you provide your services.'),
                        'name' => 'id_country',
                        'options' => array(
                            'query' => $countries,
                            'id' => 'id_country',
                            'name' => 'name',
                        ),
                        'required' => true,
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->module->l('Product'),
                        'name' => 'class_name',
                        'options' => array(
                            'query' => $products,
                            'id' => 'class_name',
                            'name' => 'name',
                        ),
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Live product code'),
                        'name' => 'product_code',
                        'class' => 'fixed-width-xl inbank_input_production',
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Test product code'),
                        'name' => 'product_code_debug',
                        'class' => 'fixed-width-xl inbank_input_debug',
                        'required' => true,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->module->l('Enable test mode'),
                        'name' => 'debug_mode',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->module->l('Enable'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->module->l('Disable'),
                            ),
                        ),
                        'class' => 'fixed-width-xl inbank_debug_mode_switcher',
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Live API key'),
                        'desc' => $this->module->l('Live API key provided by Inbank.'),
                        'name' => 'api_key',
                        'class' => 'fixed-width-xl inbank_input_production',
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Test API key'),
                        'desc' => $this->module->l('Test API key provided by Inbank.'),
                        'name' => 'api_key_debug',
                        'class' => 'fixed-width-xl inbank_input_debug',
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Live shop UUID'),
                        'desc' => $this->module->l('Live shop Unique Identifier provided by Inbank.'),
                        'name' => 'shop_uuid',
                        'class' => 'fixed-width-xl inbank_input_production',
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Test shop UUID'),
                        'desc' => $this->module->l('Test shop Unique Identifier provided by Inbank.'),
                        'name' => 'shop_uuid_debug',
                        'class' => 'fixed-width-xl inbank_input_debug',
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Localization ID'),
                        'desc' => $this->module->l('Localization ID provided by Inbank.'),
                        'name' => 'localization_id',
                        'class' => 'fixed-width-xl',
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->module->l('Localization token'),
                        'desc' => $this->module->l('Localization token provided by Inbank.'),
                        'name' => 'localization_token',
                        'class' => 'fixed-width-xl',
                        'required' => true,
                    ),
                ),
                'submit' => array(
                    'title' => $this->module->l('Next'),
                    'class' => 'btn btn-default pull-right add_product_next_btn',
                ),
                'buttons' => array(
                    array(
                        'href' => $this->context->link->getAdminLink('AdminModules').'&configure='.$this->module->name.'&tab_module'.$this->module->tab.'&module_name='.$this->module->name,
                        'title' => $this->module->l('Back to list'),
                        'icon' => 'process-icon-back',
                    ),
                ),
            ),
        );

        return $form;
    }

    public function buildPaymentForm($form, $fields_value, $submit_action = 'newInbankPayment', $param = null)
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = 'module';
        $helper->module = $this->module;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = 'id_payment';
        $helper->submit_action = $submit_action;
        $helper->back_url = $this->context->link->getAdminLink('AdminModules',
                false).'&configure='.$this->module->name.'&tab_module'.$this->module->tab.'&module_name='.$this->module->name;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules',
                false).'&configure='.$this->module->name.'&tab_module='.$this->module->tab.'&module_name='.$this->module->name.($param ? http_build_query($param) : '');
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $fields_value,
            'languages' => $this->context->controller->getLanguages(false),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(isset($form['form']) ? [$form] : $form);
    }

    private function loadSelectedPaymentForm()
    {
        $this->input_data['class_name'] = (string)Tools::getValue('class_name');
        $this->input_data['id_country'] = (int)Tools::getValue('id_country');
        $this->input_data['product_code'] = trim(Tools::getValue('product_code'));
        $this->input_data['product_code_debug'] = trim(Tools::getValue('product_code_debug'));
        $this->input_data['debug_mode'] = (bool)Tools::getValue('debug_mode');
        $this->input_data['api_key'] = trim(Tools::getValue('api_key'));
        $this->input_data['api_key_debug'] = trim(Tools::getValue('api_key_debug'));
        $this->input_data['shop_uuid'] = trim(Tools::getValue('shop_uuid'));
        $this->input_data['shop_uuid_debug'] = trim(Tools::getValue('shop_uuid_debug'));
        $this->input_data['localization_id'] = trim(Tools::getValue('localization_id'));
        $this->input_data['localization_token'] = trim(Tools::getValue('localization_token'));

        // validating all fields
        if (!$this->validateStepOne()) {
            return $this->returnToStepOneWithError($this->module->l('Missing required fields'));
        }

        //loading up the class
        $class_name = $this->input_data['class_name'];
        $class_loaded = true;
        if (!class_exists($class_name)) {
            try {
                require_once INBANK_MODULE_DIR .'src/Products/'.$class_name.'.php';
                $class_loaded = true;
            } catch (Exception $e) {
                $class_loaded = false;
            }
        }
        if ($class_loaded) {
            //checking if payment with supplied product_code exists
            $code = empty($this->input_data['product_code']) ? $this->input_data['product_code_debug'] : $this->input_data['product_code'];
            $id_country = 0;
            if (method_exists($class_name, 'getAvailableCountries')) {
                $id_country = InbankTools::getCountryIdByIso($class_name::getAvailableCountries());
            }
            if (!empty($code)) {
                $exists = InbankPaymentFormHelper::checkForDuplicate($code, $id_country, $class_name);
                if ($exists) {
                    $product_link = $this->context->link->getAdminLink(
                        'AdminModules',
                        true,
                        [],
                        [
                            'configure' => $this->module->name,
                            'module_name' => $this->module->name,
                            'tab_module' => 'payments_gateways',
                            'updateinbank_payment' => 1,
                            'id_payment' => $exists,
                        ]
                    );

                    return $this->returnToStepOneWithError(
                        $this->module->l('Payment option with this product code already exists.').
                        ' <a href="'.$product_link.'">'.$this->module->l('You can check the product here').'</a>'
                    );
                }
            }
            //creating a dummy payment to populate inbank product with data
            $payment = new InbankPayment();
            //setting debug mode here, so object constructor would pass the correct API url to API class;
            $payment->debug_mode = $this->input_data['debug_mode'];
            try {
                $product = new $class_name($payment, $this->module);
            } catch (Exception $e) {
                return $this->returnToStepOneWithError($this->module->l('Failed to create new product.'));
            }

            //filling payment option settings with data
            $payment->class_name = $class_name;
            $payment->product_code = $this->input_data['product_code'];
            $payment->product_code_debug = $this->input_data['product_code_debug'];
            $payment->id_country = $id_country;
            $payment->enabled = true;
            $payment->api_key = $this->input_data['api_key'];
            $payment->shop_uuid = $this->input_data['shop_uuid'];
            $payment->api_key_debug = $this->input_data['api_key_debug'];
            $payment->shop_uuid_debug = $this->input_data['shop_uuid_debug'];
            $payment->localization_id = $this->input_data['localization_id'];
            $payment->localization_token = $this->input_data['localization_token'];
            $payment->calc_small = $product->hasCalcSmall();

            //getting info about this payment option from api
            $product_info = $product->api->getInfo();
            if (isset($product_info->message)) {
                $error_msg = implode(',', $product_info->message);
                if (Tools::strpos($error_msg, 'product_not_found') !== false) {
                    $error_msg = $this->module->l('The product code was not found in Inbank API.');
                } else {
                    if (Tools::strpos($error_msg, 'product_code_is_missing') !== false) {
                        $error_msg = $this->module->l('Product code is missing');
                    }
                }

                return $this->returnToStepOneWithError($error_msg);
            } else {
                if (isset($product_info->error)) {
                    return $this->returnToStepOneWithError($this->module->l('Error trying to get info from Inbank API: ').implode(',',
                            $product_info->error));
                } else if($product_info === null) {
                    return $this->returnToStepOneWithError($this->module->l('Failed to connect to Inbank API'));
                }
            }
            if (isset($product_info->subtype) && $product_info->subtype !== $product->subtype) {
                return $this->returnToStepOneWithError($this->module->l('Your chosen products type doesn\'t match supplied product code type. Please choose the correct product for this product code.'));
            }
            if (!empty($product_info)) {
                foreach (Language::getLanguages(false) as $lang) {
                    $payment->title[$lang['id_lang']] = $product_info->name;
                }
                $payment->minimum_order_amount = $product_info->parameters->min_amount;
                $payment->maximum_order_amount = $product_info->parameters->max_amount;
                $payment->period = implode(",", $product_info->parameters->period_options);
                $payment->default_period = $product_info->parameters->default_period;
                $payment->max_period = max($product_info->parameters->period_options);
                $payment->default_payment_day = !empty($product_info->parameters->default_payment_day) ? $product_info->parameters->default_payment_day : 10;
                $payment->needs_approval = isset($product_info->parameters->contract_activation_requires_partner_approval) ? $product_info->parameters->contract_activation_requires_partner_approval : false;
                $payment->down_payment = isset($product_info->down_payment_enabled) ? $product_info->down_payment_enabled : false;
                $payment->api_last_upd = date('Y-m-d H:i:s');
            }

            if (!empty($payment->localization_id) && !empty($payment->localization_token)) {
                PhraseApp::fillSinglePaymentTranslations($payment);
            }

            $payment->custom_title = $payment->title;
            $payment->custom_additional_details = $payment->additional_details;

            $this->context->smarty->assign(['step' => 2]);
            $product_form = $this->enableDisabledFields($product->productApiForm());
            $product_form = $this->addDescLang($product_form, $payment);

            $product_data = $payment->toArray();

            return $this->buildPaymentForm($product_form, $product_data, 'add_inbank_payment');
        }
    }

    private function addDescLang($product_form, $payment)
    {
        //compare payment title with custom title
        $skip_titles = [];
        $skip_details = [];
        if(is_array($payment->title)) {
            foreach ($payment->title as $id_lang => $title) {
                if (isset($payment->custom_title[$id_lang]) && $payment->custom_title[$id_lang] === $title) {
                    $skip_titles[] = $id_lang;
                }
            }
        }

        if(is_array($payment->additional_details)) {
            foreach ($payment->additional_details as $id_lang => $detail) {
                if (isset($payment->custom_additional_details[$id_lang]) && $payment->custom_additional_details[$id_lang] === $detail) {
                    $skip_details[] = $id_lang;
                }
            }
        }

        foreach ($product_form['form']['input'] as $key => $input) {
            if ($input['name'] === 'custom_title') {
                $product_form['form']['input'][$key]['desc'] = $this->module->getMultilangDesc($payment->title,
                    $this->module->l('Provided by Inbank').':', "", $skip_titles);
            } else {
                if ($input['name'] === 'custom_additional_details') {
                    $product_form['form']['input'][$key]['desc'] = $this->module->getMultilangDesc($payment->additional_details,
                        $this->module->l('Provided by Inbank').':', "", $skip_details);
                }
            }
        }

        return $product_form;
    }

    private function enableDisabledFields($form)
    {
        $enable_fields = [
            'title',
            'minimum_order_amount',
            'maximum_order_amount',
            'period',
            'default_period',
            'needs_approval',
            'additional_details',
        ];
        if (is_array($form)) {
            foreach ($form['form']['input'] as $key => $input) {
                $name = $input['name'];
                if (in_array($name, $enable_fields)) {
                    $form['form']['input'][$key]['disabled'] = false;
                }
            }
        }

        return $form;
    }

    private function validateStepOne()
    {
        // checks if general fields are filled
        if (
            empty($this->input_data['class_name']) ||
            empty($this->input_data['id_country']) ||
            empty($this->input_data['localization_token']) ||
            empty($this->input_data['localization_id'])
        ) {
            return false;
        }

        // checks if debug fields are filled if debug mode is on
        if (
            $this->input_data['debug_mode'] &&
            (
                empty($this->input_data['api_key_debug']) ||
                empty($this->input_data['shop_uuid_debug']) ||
                empty($this->input_data['product_code_debug'])
            )
        ) {
            return false;
        }

        // checks if production fields are filled if debug mode is off
        if (
            !$this->input_data['debug_mode'] &&
            (
                empty($this->input_data['api_key']) ||
                empty($this->input_data['shop_uuid']) ||
                empty($this->input_data['product_code'])
            )
        ) {
            return false;
        }

        return true;
    }

    private function returnToStepOneWithError($error_message)
    {
        Media::addJsDef(['inbank_new_product_validation' => 1]);

        return $this->module->displayWarning($error_message)
            .$this->buildPaymentForm($this->newProductStep1(), $this->input_data, 'newInbankPayment');
    }

    /**
     * @param $id_payment int InbankPayment object id
     */
    public function updateProductForm($id_payment)
    {
        $paymentModel = new InbankPayment($id_payment);
        $class_name = $paymentModel->class_name;
        $product = new $class_name($paymentModel, $this->module);
        $product_form = $product->productApiForm();
        $product_form = $this->addDescLang($product_form, $paymentModel);

        $restriction_form = new InbankRestrictionForm($paymentModel, $this->module, $this->context);
        $form_fields = [$product_form, ['form' => $restriction_form->getFormFields()]];
        //setting placeholder if custom fields are empty
        /*
        if (is_array($paymentModel->custom_additional_details)) {
            foreach ($paymentModel->custom_additional_details as $id_lang => $detail) {
                if (empty($detail)) {
                    $paymentModel->custom_additional_details[$id_lang] = $paymentModel->additional_details[$id_lang];
                }
            }
        } else {
            $paymentModel->custom_additional_details = $paymentModel->additional_details;
        }

        if (is_array($paymentModel->custom_title)) {
            foreach ($paymentModel->custom_title as $id_lang => $title) {
                if (empty($title)) {
                    $paymentModel->custom_title[$id_lang] = $paymentModel->title[$id_lang];
                }
            }
        } else {
            if (empty($paymentModel->custom_title)) {
                $paymentModel->custom_title = $paymentModel->title;
            }
        }
        */

        $product_data = $product->object->toArray();
        $product_data = array_merge($product_data, $restriction_form->getFormValues());

        return $this->buildPaymentForm($form_fields, $product_data, 'updateInbankPayment');
    }
}
