<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankPaymentFormHelper
{

    private $context;
    private $module;

    public function __construct($module)
    {
        $this->context = Context::getContext();
        $this->module = $module;
    }


    public function getAllProductCountries()
    {
        //checking all inbank products and getting their countries
        $countries = [];
        $directory = INBANK_MODULE_DIR.'src/Products/';
        $scanned_directory = array_diff(scandir($directory), array('..', '.', 'index.php'));
        foreach ($scanned_directory as $file) {
            $class_name = explode('.', $file)[0];
            if (method_exists($class_name, 'getAvailableCountries')) {
                $class_countries = InbankTools::getCountryIdByIso($class_name::getAvailableCountries());
                if ($class_countries !== null) {
                    if (is_array($class_countries)) {
                        foreach ($class_countries as $country) {
                            $countries[$country] = $country;
                        }
                    } else {
                        if (is_numeric($class_countries)) {
                            $countries[$class_countries] = $class_countries;
                        }
                    }
                }
            }
        }

        if (!empty($countries)) {
            return $this->getCountryByIds($countries);
        } else {
            return [];
        }
    }

    public function getPaymentsByCountry($id_country)
    {
        //searching for all products available in selected country
        $directory = INBANK_MODULE_DIR.'src/Products/';
        $scanned_directory = array_diff(scandir($directory), array('..', '.', 'index.php'));
        $filtered_classes = [];
        foreach ($scanned_directory as $file) {
            $class_name = explode('.', $file)[0];
            if (method_exists($class_name, 'getAvailableCountries')) {
                $class_countries = InbankTools::getCountryIdByIso($class_name::getAvailableCountries());
                if ($class_countries !== null) {
                    if (is_array($class_countries) && in_array($id_country, $class_countries)) {
                        $filtered_classes[] = [
                            'class_name' => $class_name,
                            'name' => self::getProductTitle($class_name),
                        ];
                    } else {
                        if (is_numeric($class_countries) && $class_countries == $id_country) {
                            $filtered_classes[] = [
                                'class_name' => $class_name,
                                'name' => self::getProductTitle($class_name),
                            ];
                        }
                    }
                }
            }
        }

        return $filtered_classes;
    }

    private function getCountryByIds($countries)
    {
        if (is_array($countries)) {
            $countries = implode(",", $countries);
        }

        $select_countries = 'SELECT c.id_country, cl.name FROM '._DB_PREFIX_.'country c
                            LEFT JOIN '._DB_PREFIX_.'country_lang cl ON c.id_country = cl.id_country AND cl.id_lang = '.$this->context->language->id.'
                            WHERE c.id_country IN('.$countries.')
                            ORDER BY FIELD(cl.name, "Estonia", "Latvia", "Poland", "Czechia")';
        $select_countries = Db::getInstance()->executeS($select_countries);

        return $select_countries;
    }

    public function filterPayments()
    {
        $id_country = (int)Tools::getValue('id_country', false);
        //invalid id_country
        if (!is_numeric($id_country) || $id_country <= 0) {
            http_response_code(500);
            die([
                'title' => $this->module->l('Error'),
                'message' => $this->module->l("There was a problem while trying to get your selected country."),
            ]);
        }

        //searching for all products available in selected country
        $directory = INBANK_MODULE_DIR.'src/Products/';
        $scanned_directory = array_diff(scandir($directory), array('..', '.', 'index.php'));
        $filtered_classes = [];
        foreach ($scanned_directory as $file) {
            $class_name = explode('.', $file)[0];
            if (method_exists($class_name, 'getAvailableCountries')) {
                $class_countries = InbankTools::getCountryIdByIso($class_name::getAvailableCountries());
                if ($class_countries !== null) {
                    if (is_array($class_countries) && in_array($id_country, $class_countries)) {
                        $filtered_classes[$class_name] = self::getProductTitle($class_name);
                    } else {
                        if (is_numeric($class_countries) && $class_countries == $id_country) {
                            $filtered_classes[$class_name] = self::getProductTitle($class_name);
                        }
                    }
                }
            }
        }

        if (!empty($filtered_classes)) {
            $response = [];
            $response['classes'] = $filtered_classes;
            //found some classes, we can also prefill api data from other payment methods in the same country
            $api_data = 'SELECT 
                            MAX(api_key) AS api_key, MAX(api_key_debug) AS api_key_debug, MAX(shop_uuid) AS shop_uuid,
                            MAX(shop_uuid_debug) AS shop_uuid_debug
                        FROM '._DB_PREFIX_.'inbank_payment
                        WHERE id_country = '.(int)$id_country;
            $api_data2 = 'SELECT 
                            MAX(localization_id) AS localization_id, MAX(localization_token) AS localization_token
                        FROM '._DB_PREFIX_.'inbank_payment';
            $api_data = Db::getInstance()->getRow($api_data);
            $api_data2 = Db::getInstance()->getRow($api_data2);
            $api_data = array_merge($api_data, $api_data2);
            if (!empty($api_data)) {
                $response['api_data'] = $api_data;
                //message to inform the user about prefilled fields
                $response['title'] = $this->module->l('Notice');
                $response['message'] = $this->module->l('We found another Product that matches your selected country. We prefilled some fields for your convenience.');
            } else {
                $response['api_data'] = false;
            }

            return json_encode($response);
        } else {
            http_response_code(500);

            return json_encode([
                'title' => $this->module->l('Error'),
                'message' => $this->module->l("There are no payments available in your selected country."),
            ]);
        }
    }

    private static function getProductTitle($class_name)
    {
        $file_path = INBANK_MODULE_DIR."src/Products/".$class_name.".php";

        if (!class_exists($class_name) && file_exists($file_path)) {
            try {
                require_once($file_path);
            } catch (Exception $e) {
                InbankLogs::addLog(
                    0,
                    'Failed to load class '.$class_name.'. InbankPaymentFormHelper.php:getProductTitle. '.$e->getMessage()
                );
            }
        }
        if (class_exists($class_name)) {
            if (method_exists($class_name, "getDefaultTitle")) {
                return $class_name::getDefaultTitle();
            }
        }

        return $class_name;
    }

    /**
     * Checks for duplicate product code
     * @param  string  $product_code
     * @param  int  $id_country
     * @param  string  $class_name
     * @param  int  $self_id_payment
     * @return int
     */
    public static function checkForDuplicate($product_code, $id_country, $class_name, $self_id_payment = 0)
    {
        $exists = 'SELECT id_payment FROM '._DB_PREFIX_.'inbank_payment 
                    WHERE (product_code = "'.pSQL($product_code).'" 
                    OR product_code_debug = "'.pSQL($product_code).'") 
                    AND id_country = '.(int)$id_country.' 
                    AND class_name = "'.pSQL($class_name).'"';
        if ($self_id_payment !== 0) {
            $exists .= ' AND id_payment <> '.$self_id_payment;
        }

        return (int)Db::getInstance()->getValue($exists);
    }

    /**
     * Logo uploaded
     *
     * @param $id
     * @param $name
     * @return bool
     * @throws PrestaShopException
     */
    public static function uploadImage($id, $name)
    {
        if (isset($_FILES[$name]['tmp_name']) && !empty($_FILES[$name]['tmp_name'])) {
            // Delete old image
            if (!file_exists(INBANK_IMG_DIR)) // Check image validity
            {
                mkdir(INBANK_IMG_DIR);
            }

            if ($error = ImageManager::validateUpload($_FILES[$name], Tools::getMaxUploadSize())) {
                $errors[] = $error;
            }

            $tmp_name = tempnam(_PS_TMP_IMG_DIR_, 'PS');
            if (!$tmp_name) {
                return false;
            }

            if (!move_uploaded_file($_FILES[$name]['tmp_name'], $tmp_name)) {
                return false;
            }

            // Evaluate the memory required to resize the image: if it's too much, you can't resize it.
            if (!ImageManager::checkImageMemoryLimit($tmp_name)) {
                $errors[] = Tools::displayError('Due to memory limit restrictions, this image cannot be loaded. Please increase your memory_limit value via your server\'s configuration settings. ');
            }

            list($tmpWidth, $tmpHeight, $type) = getimagesize($tmp_name);
            $dest_height_small = 26;
            $dest_width_small = (int)($tmpWidth / ($tmpHeight / $dest_height_small));
            $dest_height_medium = 36;
            $dest_width_medium = (int)($tmpWidth / ($tmpHeight / $dest_height_medium));

            // Copy new image
            if (empty($errors) &&
                (!ImageManager::resize($tmp_name, INBANK_IMG_DIR . $id . '_small.jpg', $dest_width_small, $dest_height_small) ||
                !ImageManager::resize($tmp_name, INBANK_IMG_DIR . $id . '_medium.jpg', $dest_width_medium, $dest_height_medium))
            ) {
                $errors[] = Tools::displayError('An error occurred while uploading the image.');
            }

            unlink($tmp_name);

            if (count($errors)) {
                return $errors;
            } else {
                return true;
            }
        }
        return true;
    }
}
