<?php

class InbankRestrictionForm
{
    const TYPE_AND = 0;
    const TYPE_OR = 1;

    /** @var InbankPayment */
    private $inbank_payment;
    /** @var Inbank */
    private $module;
    /** @var Context */
    private $context;

    public function __construct($inbank_payment, $module, $context)
    {
        $this->inbank_payment = $inbank_payment;
        $this->module = $module;
        $this->context = $context;
    }

    public function getFormValues()
    {
        return [
            'categories' => $this->inbank_payment->getCategoryRestrictions(),
            'products' => $this->inbank_payment->getProductRestrictions(),
            'restriction_type' => $this->inbank_payment->restriction_type,
            'product_autocomplete' => ''
        ];
    }

    public function getFormFields()
    {
        return [
            'legend' => [
                'title' => $this->module->l('Product restrictions'),
                'icon' => 'icon-cogs'
            ],
            'input' => [
                [
                    'type' => 'categories',
                    'label' => $this->module->l('Category restrictions'),
                    'name' => 'categories',
                    'desc' => $this->module->l('Only allow this payment to selected categories'),
                    'tree' => array(
                        'id' => 'categories-tree',
                        'root_category' => 2,
                        'use_search' => true,
                        'use_checkbox' => true,
                        'selected_categories' => $this->inbank_payment->getCategoryRestrictions(),
                    ),
                ],
                [
                    'type' => 'text',
                    'label' => $this->module->l('Product search'),
                    'name' => 'product_autocomplete',
                    'class' => 'product_search'
                ],
                [
                    'type' => 'html',
                    'label' => $this->module->l('Product restrictions'),
                    'html_content' => $this->getProductRestrictionTable(),
                    'name' => ''
                ],
                [
                    'type' => 'select',
                    'label' => $this->module->l('Restriction rule'),
                    'desc' => $this->module->l('Use this for cases when both category and product restrictions are applied'),
                    'name' => 'restriction_type',
                    'options' => array(
                        'query' => [
                            [
                                'id' => self::TYPE_AND,
                                'name' => $this->module->l('AND, products must be present in selected category and product restrictions')
                            ],
                            [
                                'id' => self::TYPE_OR,
                                'name' => $this->module->l('OR, products can be present in selected category or product restrictions')
                            ]
                        ],
                        'id' => 'id',
                        'name' => 'name',
                    ),
                ]
            ],
            'submit' => [
                'title' => $this->module->l('Save'),
                'class' => 'btn btn-default pull-right'
            ]
        ];
    }

    public function saveRestrictions()
    {
        $categories = array_map('intval', Tools::getValue('categories', []));
        $products = array_map('intval', Tools::getValue('products', []));

        return $this->inbank_payment->syncCategoryRestrictions($categories)
            && $this->inbank_payment->syncProductRestrictions($products);
    }

    private function getProductRestrictionTable()
    {
        $products = Db::getInstance()->executeS(
            'SELECT p.id_product, pl.name, p.reference FROM '._DB_PREFIX_.'product p
            INNER JOIN '._DB_PREFIX_.'product_lang pl ON pl.id_product = p.id_product
            INNER JOIN '._DB_PREFIX_.'inbank_product_restriction ipr ON ipr.id_product = p.id_product
            WHERE ipr.id_payment = '.$this->inbank_payment->id.' AND pl.id_lang = '.$this->context->language->id
        );

        $this->context->smarty->assign([
            'products' => $products
        ]);

        return $this->context->smarty->fetch(
            _PS_MODULE_DIR_.$this->module->name.'/views/templates/admin/product_restriction_table.tpl'
        );
    }

}