<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankUpdater
{
    public static function check($module_name, $module_version, $system_version)
    {
        $content = self::getNewestVersion($module_name, $module_version, $system_version);
        $new_version = self::getConfig(Tools::strtolower($module_name.'_last_update_check_version'), $module_version);

        if (version_compare($new_version, $module_version, '>')) {
            $response_json = self::getConfig(Tools::strtolower($module_name.'_last_update_check_response'), '');

            if (!empty($response_json)) {
                $response = json_decode($response_json, true);
                $docs_url = $response['docs_url'];
                $update_url = $response['update_url'];
                $additional_text = base64_decode($response['additional_text_base64']);

                $linkToDownload = $readDocumentation = '';
                if (isset($update_url) && !empty($update_url)) {
                    $linkToDownload = self::getString(1).' - <a target="_blank" href="'.$update_url.'">'.$update_url.'</a>.';
                }

                if (isset($docs_url) && !empty($docs_url)) {
                    $readDocumentation = self::getString(2).' <a target="_blank" href="'.$docs_url.'">'.self::getString(3).'</a>.';
                }

                $content .= self::displayWarning(self::getString(4).' '.$linkToDownload.' '.$readDocumentation.' '.$additional_text);
            }
        }

        return $content;
    }

    private static function getNewestVersion($module_name, $module_version, $system_version)
    {
        try {
            if (self::getConfig(Tools::strtolower($module_name.'_last_update_check_date'), 0) < (date("U") - 86400)) {
                self::updateConfig(Tools::strtolower($module_name.'_last_update_check_date'), date("U"));

                if (ini_get("allow_url_fopen")) {
                    if (function_exists("file_get_contents")) {
                        $ctx = stream_context_create(array(
                            'http' => array(
                                'timeout' => 10,  // timeout 10 sec
                            ),
                        ));

                        $response_json = @Tools::file_get_contents('https://developer.inbank.dev/payment-modules/updater/get.php?fw='.self::getModuleFramework().'&m='.$module_name."&v=".self::encrypt($module_version)."&pv=".self::encrypt($system_version)."&h=".self::encrypt(self::getUrl()),
                            false, $ctx);

                        if (!empty($response_json)) {
                            $response = json_decode($response_json, true);

                            if (isset($response['new_version']) && !empty($response['new_version'])) {
                                self::updateConfig(Tools::strtolower($module_name.'_last_update_check_version'),
                                    $response['new_version']);
                                self::updateConfig(Tools::strtolower($module_name.'_last_update_check_response'),
                                    $response_json);

                                if (version_compare($module_version, $response['new_version'], '=')) {
                                    return self::displayConfirmation(self::getString(5));
                                }
                            }
                        }
                    } else {
                        return self::displayWarning(self::getString(6));
                    }
                } else {
                    return self::displayWarning(self::getString(7));
                }
            }

            return '';
        } catch (Exception $ex) {
            self::updateConfig(Tools::strtolower($module_name.'_last_update_check_date'), date("U"));

            return self::displayWarning(self::getString(8).' '.$ex->getMessage());
        }
    }

    private static function encrypt($input)
    {
        return strtr(base64_encode($input), '+/=', '._-');
    }

    // functions witch need to be changed by the framework

    private static function displayConfirmation($text)
    {
        $output = '
        <div class="bootstrap">
        <div class="module_confirmation conf confirm alert alert-success">
            <button type="button" class="close" data-dismiss="alert">&times;</button>
            '.$text.'
        </div>
        </div>';

        return $output;
    }

    private static function displayWarning($text)
    {
        $output = '
        <div class="bootstrap">
        <div class="module_warning alert alert-warning" >
            <button type="button" class="close" data-dismiss="alert">&times;</button>
            '.$text.'
        </div>
        </div>';

        return $output;
    }

    private static function getConfig($key, $default_value)
    {
        return Configuration::get(Tools::strtoupper($key), $default_value);
    }

    private static function updateConfig($key, $value)
    {
        return Configuration::updateValue(Tools::strtoupper($key), $value);
    }

    private static function getUrl()
    {
        return _PS_BASE_URL_;
    }

    private static function getModuleFramework()
    {
        return 'prestashop';
    }

    private static function getString($id_string)
    {
        $strings = [
            1 => 'Updated module can be downloaded from',
            2 => 'More about update',
            3 => 'read here',
            4 => 'New module update was found.',
            5 => 'Successfully checked for updates. No new updates were found. You are using the latest module version.',
            6 => 'Unable to check for updates. Funcion `file_get_contents` is disabled.',
            7 => 'Unable to check for updates. Setting `allow_url_fopen` is disabled.',
            8 => 'Failed to check for updates. Error:',
        ];

        return isset($strings[$id_string]) ? $strings[$id_string] : '';
    }
}
