<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankConfigurator
{

    /**
     * @var Inbank
     */
    private $module;
    private $config;
    private $config_dir;

    public function __construct($module)
    {
        $this->module = $module;
        $this->config_dir = INBANK_MODULE_DIR.'configuration.json';
    }

    public function autoConfig()
    {
        if (file_exists($this->config_dir)) {
            try {
                $this->config = json_decode(Tools::file_get_contents($this->config_dir));
            } catch (Exception $e) {
                $this->addLog('Failed to run auto configuration. '.$e->getMessage());

                return;
            }
        }

        if (empty($this->config)) {
            return;
        }

        $this->fillPayments();
        $this->removeConfig();
        $this->fillPaymentData();

        if (is_file($this->config_dir)) {
            $this->addLog('Unable to delete configuration file from ('.$this->config_dir.'). Please delete it manually.');
        }
    }

    private function fillPayments()
    {
        if (!isset($this->config->products)) {
            $this->addLog('Configuration has no products.');

            return;
        }

        foreach ($this->config->products as $product) {
            $country_iso = $product->country_iso;
            // trying to get country id
            $id_country = Country::getByIso((string)$country_iso);
            if (!$id_country) {
                $this->addLog('Failed to create product. Country with ISO code ('.(string)$country_iso.') not found');
                continue;
            }
            $subtype = $product->subtype;

            //trying to get class_name for this product
            $class_name = $this->findProduct($id_country, $subtype);
            if (!$class_name) {
                $this->addLog('Failed to create product. Found none or multiple matching Inbank Products.');
                continue;
            }

            $inbank_product = new InbankPayment();
            $inbank_product->enabled = isset($product->enabled) ? $product->enabled : false;
            $inbank_product->class_name = $class_name;
            $inbank_product->id_country = (int)$id_country;
            $inbank_product->product_code = isset($product->product_code) ? $product->product_code : '';
            $inbank_product->product_code_debug = isset($product->product_code_debug) ? $product->product_code_debug : '';
            $inbank_product->calc_small = isset($product->calc_small) ? $product->calc_small : false;
            $inbank_product->calc_large = isset($product->calc_large) ? $product->calc_large : false;
            if (isset($product->calc_small_position) && !empty($product->calc_small_position)) {
                $inbank_product->calc_small_position = $product->calc_small_position;
            }
            $inbank_product->api_key = isset($product->api_key) ? $product->api_key : '';
            $inbank_product->api_key_debug = isset($product->api_key_debug) ? $product->api_key_debug : '';
            $inbank_product->shop_uuid = isset($product->shop_uuid) ? $product->shop_uuid : '';
            $inbank_product->shop_uuid_debug = isset($product->shop_uuid_debug) ? $product->shop_uuid_debug : '';
            $inbank_product->localization_id = isset($product->localization_id) ? $product->localization_id : '';
            $inbank_product->localization_token = isset($product->localization_token) ? $product->localization_token : '';
            $inbank_product->logo_size = 'medium';
            if (isset($product->use_logo) && !empty($product->use_logo)) {
                $inbank_product->use_logo = $product->use_logo;
            }
            $inbank_product->debug_mode = 0;
            if (!empty($inbank_product->api_key) && !empty($inbank_product->shop_uuid)) {
                $inbank_product->debug_mode = 0;
            } else {
                if (!empty($inbank_product->api_key_debug) && !empty($inbank_product->shop_uuid_debug)) {
                    $inbank_product->debug_mode = 1;
                }
            }

            try {
                $inbank_product->save();
            } catch (Exception $e) {
                $this->addLog('Failed to save product created from configuration. '.$e->getMessage());
                continue;
            }
        }
    }

    private function removeConfig()
    {
        if (is_file($this->config_dir)) {
            //trying to delete file
            if (!unlink($this->config_dir)) {
                //failed to delete, try to replace it's content with an empty string
                file_put_contents($this->config_dir, '');
            }
        }
    }

    private function fillPaymentData()
    {
        $this->module->updateAllProducts();
    }

    private function findProduct($id_country, $subtype)
    {
        $products = [];
        $directory = INBANK_MODULE_DIR.'src/Products/';
        $scanned_directory = array_diff(scandir($directory), array('..', '.', 'index.php'));
        foreach ($scanned_directory as $file) {
            $class_name = explode('.', $file)[0];
            if (method_exists($class_name, 'getAvailableCountries') && method_exists($class_name, 'getSubtype')) {
                $class_countries = InbankTools::getCountryIdByIso($class_name::getAvailableCountries());
                $class_subtype = $class_name::getSubtype();
                if ($class_countries && $class_subtype) {
                    if (is_array($class_countries)) {
                        if (in_array($id_country, $class_countries) && $class_subtype == $subtype) {
                            $products[] = $class_name;
                        }
                    } else {
                        if (is_numeric($class_countries) && $class_countries == $id_country && $class_subtype == $subtype) {
                            $products[] = $class_name;
                        }
                    }
                }
            }
        }
        if (!empty($products) && count($products) === 1) {
            return reset($products);
        } else {
            return false;
        }
    }

    private function addLog(
        $message,
        $type = InbankLogs::TYPE_ERROR,
        $product_code = "",
        $endpoint = "",
        $response = ""
    ) {
        InbankLogs::addLog(0, $message, $type, $product_code, $endpoint, $response);
    }
}