<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankInstaller
{

    private $module;
    //estonia, poland, latvia, czech
    const AVAILABLE_COUNTRIES = ['EE', 'PL', 'LV', 'CZ'];
    //euro, zloty
    const AVAILABLE_CURRENCIES = ['EUR', 'PLN', 'CZK'];

    public function __construct($module)
    {
        $this->module = $module;
    }

    public function runInstaller()
    {
        $res = true;
        $res &= $this->registerHooks();
        $res &= $this->createOrderStatuses();
        $res &= $this->installDatabase();
        $res &= $this->registerTabs();
        $res &= $this->addCountryRestrictions();
        $res &= $this->addCurrencyRestrictions();

        // install successful, check for configuration file
        if ($res) {
            $configurator = new InbankConfigurator($this->module);
            $configurator->autoConfig();
        }

        return $res;
    }

    public function runUninstaller()
    {
        $res = true;
        $res &= $this->uninstallDatabase();
        $res &= $this->deleteTabs();

        array_map('unlink', glob(INBANK_IMG_DIR."*"));
        @rmdir(INBANK_IMG_DIR);

        return $res;
    }

    private function registerHooks()
    {
        $res = true;
        $res &= $this->module->registerHook('displayProductPriceBlock'); // used to display inbank leasing options
        $res &= $this->module->registerHook('paymentOptions');
        $res &= $this->module->registerHook('payment');
        $res &= $this->module->registerHook('displayLeasing');
        $res &= $this->module->registerHook('displayProductAdditionalInfo');
        $res &= $this->module->registerHook('displayReassurance');
        $res &= $this->module->registerHook('adminOrder');
        $res &= $this->module->registerHook('displayHeader');
        $res &= $this->module->registerHook('header');
        $res &= $this->module->registerHook('displayBackOfficeHeader');
        $res &= $this->module->registerHook('displayPaymentReturn');
        $res &= $this->module->registerHook('displayProductDeliveryTime');

        return $res;
    }

    private function createOrderStatuses()
    {
        $statuses = [
            'INBANK_PENDING' => ['name' => 'Inbank: waiting for financing', 'color' => '#34209E', 'mail' => false, 'paid' => false],
            'INBANK_CANCELLED' => ['name' => 'Inbank: cancelled', 'color' => '#DC143C', 'mail' => false, 'paid' => false],
            'INBANK_DECLINED' => ['name' => 'Inbank: credit declined', 'color' => '#DC143C', 'mail' => false, 'paid' => false],
            'INBANK_EXPIRED' => ['name' => 'Inbank: payment session expired', 'color' => '#8F0621', 'mail' => false, 'paid' => false],
            'INBANK_GRANTED' => ['name' => 'Inbank: credit granted. Action required.', 'color' => '#01B887', 'mail' => false, 'paid' => false],
            'INBANK_COMPLETED' => ['name' => 'Inbank: purchase financed', 'color' => '#32CD32', 'mail' => false, 'paid' => true],
        ];

        $res = true;
        foreach ($statuses as $key => $info) {
            if (!Configuration::get($key)) {
                $res &= $this->createOrderStatus($info['name'], $info['color'], $info['mail'], $info['paid'], $key);
            }
        }

        return $res;
    }

    private function createOrderStatus($name, $color, $sendMail, $paid, $configurationValue)
    {
        $order_pending = new OrderState();
        $order_pending->name = $this->createMultiLangField($name);
        $order_pending->send_email = $sendMail;
        $order_pending->invoice = 0;
        $order_pending->color = $color;
        $order_pending->unremovable = false;
        $order_pending->logable = $paid;
        $order_pending->paid = $paid;
        $order_pending->add();
        $id_order_state = $order_pending->id;

        if (!$id_order_state) {
            return false;
        } else {
            Configuration::updateValue($configurationValue, $id_order_state);

            return true;
        }
    }

    protected function createMultiLangField($field)
    {
        $languages = Language::getLanguages(false);

        $res = array();
        foreach ($languages as $lang) {
            $res[$lang['id_lang']] = $field;
        }

        return $res;
    }

    /**
     * Install SQL file
     *
     * @return bool Module SQL installed successfully
     */
    private function installDatabase()
    {
        $sqlStatements = [];
        $installSqlFileName = INBANK_MODULE_DIR.'sql/install.php';
        require ($installSqlFileName);

        if (empty($sqlStatements)) {
            return true;
        }

        $result = true;
        foreach ($sqlStatements as $sql) {
            $sql = str_replace('PREFIX_', _DB_PREFIX_, $sql);
            $sql = str_replace('ENGINE_TYPE', _MYSQL_ENGINE_, $sql);
            $result &= Db::getInstance()->execute($sql);
        }

        return $result;
    }

    /**
     * Uninstall module SQL files
     *
     * @return bool Module SQL Uninstalled successfully
     */
    private function uninstallDatabase()
    {
        $sqlStatements = [];
        $uninstallSqlFileName = INBANK_MODULE_DIR.'sql/uninstall.php';
        require ($uninstallSqlFileName);

        if (empty($sqlStatements)) {
            return true;
        }

        $result = true;
        foreach ($sqlStatements as $sql) {
            $sql = str_replace('PREFIX_', _DB_PREFIX_, $sql);
            $sql = str_replace('ENGINE_TYPE', _MYSQL_ENGINE_, $sql);
            $result &= Db::getInstance()->execute($sql);
        }

        return $result;
    }

    /**
     * Register tabs
     *
     * @return bool true if registration successful
     */
    private function registerTabs()
    {
        if ($this->module->is17) {
            $tab = new Tab();
            $tab->id_parent = Tab::getIdFromClassName('AdminParentPayment');
            $tab->module = $this->module->name;
            $tab->name = $this->createMultiLangField("Inbank Settings");
            $tab->class_name = 'AdminInbankRedirect';

            return $tab->save();
        } else {
            $tab = new Tab();
            $tab->id_parent = Tab::getIdFromClassName('AdminParentModules');
            $tab->module = $this->module->name;
            $tab->name = $this->createMultiLangField("Inbank Settings");
            $tab->class_name = 'AdminInbankRedirect';

            return $tab->save();
        }
    }

    private function deleteTabs()
    {
        $id_tab = Tab::getIdFromClassName("AdminInbankRedirect");
        if ($id_tab > 0) {
            $tab = new Tab($id_tab);

            return $tab->delete();
        }

        return true;
    }

    private function addCurrencyRestrictions()
    {
        //remove restrictions added on install by parent install
        Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'module_currency` WHERE id_module = '.(int)$this->module->id);
        $shops = Shop::getShops();
        $res = true;
        foreach ($shops as $shop) {
            foreach (self::AVAILABLE_CURRENCIES as $CURRENCY) {
                $CURRENCY = pSQL(Tools::strtoupper($CURRENCY));
                $id_shop = (int)$shop['id_shop'];
                $id_currency = (int)Currency::getIdByIsoCode($CURRENCY, $id_shop);
                if ($id_currency) {
                    $res &= Db::getInstance()->insert(
                        'module_currency',
                        ['id_module' => $this->module->id, 'id_shop' => $id_shop, 'id_currency' => $id_currency],
                        false,
                        true,
                        Db::INSERT_IGNORE
                    );
                }
            }
        }

        return $res;
    }

    public function addCountryRestrictions()
    {
        //remove restrictions added on install by parent install
        Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'module_country` WHERE id_module = '.(int)$this->module->id);
        $countries_formatted = [];
        foreach (self::AVAILABLE_COUNTRIES as $COUNTRY) {
            $id_country = (int)Db::getInstance()->getValue('SELECT id_country FROM '._DB_PREFIX_.'country WHERE iso_code = "'.$COUNTRY.'"');
            if($id_country) {
                $countries_formatted[] = ['id_country' => $id_country];
            }
        }

        //add countries
        return Country::addModuleRestrictions([], $countries_formatted, [['id_module' => $this->module->id]]);
    }

    /**
     * Get module SQL content
     *
     * @return string Module SQL content
     */
    protected function getSqlStatements($fileName)
    {
        $sqlStatements = Tools::file_get_contents($fileName);
        $sqlStatements = str_replace('PREFIX_', _DB_PREFIX_, $sqlStatements);
        $sqlStatements = str_replace('ENGINE_TYPE', _MYSQL_ENGINE_, $sqlStatements);

        return $sqlStatements;
    }

}
