<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankHirePurchaseLV extends InbankBalticsProduct
{

    private $class_name = 'InbankHirePurchaseLV';
    public $production_api = 'https://api.inbank.lv/partner/v2/';
    public $debug_api = 'https://demo-api.inbank.lv/partner/v2/';
    public $production_epos = 'https://epos.inbank.lv/';
    public $debug_epos = 'https://demo-epos.inbank.lv/';
    public $has_large_calc = true;
    public $default_logo_name = 'inbank-logo.svg';
    public $subtype = 'hire_purchase';
    const INBANK_CALC_SMALL = 'calc_small_w_sample';
    const INBANK_CALC_SMALL_DOWNPAYMENT = 'calc_small_w_downpayment_sample';
    const INBANK_PAYMENT = 'payment_representative';
    const INBANK_DOWNPAYMENT = 'downpayment_representative';


    public static function getDefaultTitle()
    {
        return "Inbank Hire Purchase";
    }

    public static function getAvailableCountries()
    {
        return 'lv';
    }

    public static function getSubtype()
    {
        return 'hire_purchase';
    }

    public static function getPhraseTemplateVars($template, $additional_info = [])
    {
        $vars = [];
        //$vars['title'] = "presta_plugin_product_hp_marketingMessage";
        $vars['local'] = [
            'additional_details' => 'presta_plugin_product_hp_consent',
            'granted' => 'presta_plugin_product_granted_text',
            'representative_example' => 'presta_plugin_representative_example',
            'euribor_1m' => 'presta_representative_text_EURIBOR_1M',
            'euribor_3m' => 'presta_representative_text_EURIBOR_3M',
            'euribor_6m' => 'presta_representative_text_EURIBOR_6M',
            'euribor_12m' => 'presta_representative_text_EURIBOR_12M'
        ];
        switch ($template) {
            case self::INBANK_DOWNPAYMENT:
            case self::INBANK_PAYMENT:
                $vars['body'] = [
                    'product_granted_text' => 'presta_plugin_product_hp_marketingMessage',
                    'down_payment' => 'presta_plugin_downpayment_message',
                    'selling_point_1' => 'presta_plan_selling_point_one',
                    'selling_point_2' => 'presta_plan_selling_point_two',
                    'selling_point_3' => 'presta_plan_selling_point_three',
                ];
                break;
            case self::INBANK_CALC_SMALL:
            case self::INBANK_CALC_SMALL_DOWNPAYMENT:
                $vars['body'] = [
                    'marketing_message' => 'presta_plugin_product_hp_marketingMessage',
                    'down_payment' => 'presta_plugin_downpayment_message'
                ];
                break;
        }

        return $vars;
    }

    //styling payment option for prestashop 1.6
    public function displayPaymentOption16($product_price, $currency_code)
    {
        $calculations = $this->api->getCreditCalculation(
            ['amount' => $product_price, 'response_level' => 'advanced']
        );
        // we validate the api info
        if (!isset($calculations->product_code) || $calculations->product_code !== $this->getProductCode()) {
            return false;
        }
        $title = self::getDefaultTitle();
        $placeholders = [
            'default_period' => $this->object->max_period
        ];
        if (!empty($this->object->custom_title)) {
            $title = self::replacePlaceholders($this->object->custom_title, $placeholders);
        } elseif (!empty($this->object->title)) {
            $title = self::replacePlaceholders($this->object->title, $placeholders);
        }
        $context = Context::getContext();
        $context->smarty->assign([
            'payment_title' => $title,
            'payment_class' => $this->object->class_name,
            'payment_image' => $this->getLogoPath(),
            'payment_id' => $this->object->id,
            'use_logo' => $this->object->use_logo,
        ]);

        return $context->smarty->fetch(INBANK_MODULE_DIR.'views/templates/hook/payment_16.tpl');
    }


    public function getPaymentTemplate($product_price, $calculations = [], &$placeholders = null)
    {
        if (!$calculations) {
            $calculations = $this->api->getCreditCalculation(
                ['amount' => $product_price, 'response_level' => 'advanced']
            );
        }
        $template = $this->object->down_payment ? self::INBANK_DOWNPAYMENT : self::INBANK_PAYMENT;
        $path = INBANK_MODULE_DIR.'cache/'.$this->object->id.'_'.(int)$this->object->debug_mode.'_'.$template.'_'
            .Context::getContext()->language->id.'.html';

        $currency_symbol = $this->getCurrencySymbolByCode($calculations->currency);
        $monthly_payment = number_format(round($calculations->payment_amount_monthly, 2), 2, '.', '') . ' ' . $currency_symbol;

        $placeholders = [
            'checkbox_image' => INBANK_MODULE_URL.'views/img/checkbox-inbank.png',
            'monthlyPayment' => $monthly_payment,
            'loan_amount' => number_format($calculations->amount, 2),
            'loan_period' => $this->object->max_period,
            'loan_interest_rate' => $calculations->interest_rate_annual*100,
            'loan_commission' => number_format($calculations->advanced->contract_fee_amount_total, 2),
            'loan_admin_fee' => number_format($calculations->advanced->admin_fee_amount_monthly, 2),
            'loan_total_amount' => number_format($calculations->advanced->repayments_amount_total, 2),
            'loan_apr' => $calculations->credit_cost_rate_annual*100,
            'additional_details' => $this->object->custom_additional_details,
            'representative_example' => $this->object->representative_example
        ];

        if ($calculations->advanced->interest_type == 'floating') {
            $base_rate_type = strtolower($calculations->advanced->floating_interest_rate->base_rate_type);
            if(isset($this->object->{$base_rate_type}) && !empty($this->object->{$base_rate_type})) {
                $placeholders['representative_example'] = $this->object->{$base_rate_type};
            }
            $placeholders['margin'] = round((float)$calculations->advanced->floating_interest_rate->margin * 100, 2);
            $placeholders['base_rate_value'] = round((float)$calculations->advanced->floating_interest_rate->base_rate_value * 100, 2);
        }

        if ($this->object->down_payment) {
            $placeholders['min_down_payment'] = number_format(
                (float)($calculations->down_payment_minimum_amount ?? 0),
                2
                ) . ' ' . $currency_symbol;
        }

        $generated_template = "";
        if (file_exists($path)) {
            $generated_template = self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
        } else {
            $this->generateTemplate($template);
            if (file_exists($path)) {
                $generated_template = self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
            }
        }

        return !empty($generated_template) ? self::replacePlaceholders($generated_template, $placeholders) : false;
    }

    public function getCalcSmallTemplate($product_price)
    {
        $template = $this->object->down_payment ? self::INBANK_CALC_SMALL_DOWNPAYMENT : self::INBANK_CALC_SMALL;
        $path = INBANK_MODULE_DIR.'cache/'.$this->object->id.'_'.(int)$this->object->debug_mode.'_'.$template.'_'.
            Context::getContext()->language->id.'.html';
        $api_response = $this->api->getCreditCalculation(
            ['amount' => $product_price, 'response_level' => 'advanced']
        );
        if (!isset($api_response->payment_amount_monthly)) {
            return false;
        }
        $currency_symbol = $this->getCurrencySymbolByCode($api_response->currency);
        $placeholders = [
            'monthlyPayment' => number_format(round($api_response->payment_amount_monthly, 2), 2, '.',
                    '') . ' ' . $currency_symbol,
            'loan_amount' => number_format($api_response->amount, 2),
            'loan_period' => $this->object->max_period,
            'loan_interest_rate' => $api_response->interest_rate_annual * 100,
            'loan_commission' => number_format($api_response->advanced->contract_fee_amount_total, 2),
            'loan_admin_fee' => number_format($api_response->advanced->admin_fee_amount_monthly, 2),
            'loan_total_amount' => number_format($api_response->advanced->repayments_amount_total, 2),
            'loan_apr' => $api_response->credit_cost_rate_annual * 100,
            'representative_example' => $this->object->representative_example
        ];

        if ($api_response->advanced->interest_type == 'floating') {
            $base_rate_type = strtolower($api_response->advanced->floating_interest_rate->base_rate_type);
            if(isset($this->object->{$base_rate_type}) && !empty($this->object->{$base_rate_type})) {
                $placeholders['representative_example'] = $this->object->{$base_rate_type};
            }
            $placeholders['margin'] = round((float)$api_response->advanced->floating_interest_rate->margin * 100, 2);
            $placeholders['base_rate_value'] = round((float)$api_response->advanced->floating_interest_rate->base_rate_value * 100, 2);
        }

        if ($this->object->down_payment) {
            $placeholders['min_down_payment'] = number_format(
                (float)($api_response->down_payment_minimum_amount ?? 0),
                2
                ) . ' ' . $currency_symbol;
        }

        if ($this->object->use_logo) {
            $logo = $this->getLogo();
            $placeholders['image_link'] = '<img class="inbank_logo_product" src="'.$logo['url'].'" style="'.$logo['style'].'"/>';
        } else {
            $placeholders['image_link'] = '';
        }

        $generated_template = "";
        if (file_exists($path)) {
            $generated_template = self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
        } else {
            $this->generateTemplate($template);
            if (file_exists($path)) {
                $generated_template = self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
            }
        }

        return !empty($generated_template) ? self::replacePlaceholders($generated_template, $placeholders) : null;
    }

    public function displayPaymentOption17($product_price, $currency_code)
    {
        if (!$this->isAcceptedCurrency('EUR', $currency_code)) {
            return false;
        }
        $calculations = $this->api->getCreditCalculation(
            ['amount' => $product_price, 'currency' => $currency_code, 'response_level' => 'advanced']
        );
        if (!isset($calculations->product_code) && !isset($calculations->total_cost)) {
            return false;
        }
        $placeholders = [];
        $paymentOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $paymentOption->setModuleName($this->module->name);
        if ($this->object->use_logo) {
            $paymentOption->setLogo($this->getLogoPath(true));
        }
        /*
        $title = self::getDefaultTitle();
        if (!empty($this->object->custom_title)) {
            $title = self::replacePlaceholders($this->object->custom_title, $placeholders);
        } else if (!empty($this->object->title)) {
            $title = self::replacePlaceholders($this->object->title, $placeholders);
        }*/
        $title = self::replacePlaceholders($this->object->custom_title, $placeholders);
        $paymentOption->setCallToActionText($title);
        $paymentOption->setAction(Context::getContext()->link->getModuleLink(
            $this->module->name,
            'validation',
            [
                'inbank_payment' => $this->object->id,
            ],
            true
        ));
        $paymentOption->setAdditionalInformation(
            $this->getPaymentTemplate($product_price, $calculations)
        );

        return $paymentOption;
    }
}
