<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankIndivyGoEE extends InbankBalticsProduct
{

    private $class_name = 'InbankIndivyGoEE';
    public $production_api = 'https://api.inbank.ee/partner/v2/';
    public $debug_api = 'https://demo-api.inbank.ee/partner/v2/';
    public $default_logo_name = 'inbank-logo.svg';
    public $subtype = 'invoice';

    public static function getDefaultTitle()
    {
        return 'Inbank Pay next month';
    }

    public static function getAvailableCountries()
    {
        return 'ee';
    }

    public static function getSubtype()
    {
        return 'invoice';
    }

    public static function getPhraseTemplateVars($template, $additional_info = [])
    {
        $vars = [];
        $vars['title'] = "presta_plugin_product_go_marketingMessage_period";
        $vars['additional_details'] = 'presta_plugin_product_go_consent';
        $vars['granted'] = 'presta_plugin_product_granted_text';
        switch ($template) {
            case self::INBANK_PAYMENT:
                $vars['body'] = [
                    'down_payment' => 'presta_plugin_downpayment_message',
                    'selling_point_1' => 'presta_go_selling_point_one',
                    'selling_point_2' => 'presta_go_selling_point_two',
                    'selling_point_3' => 'presta_go_selling_point_three',
                ];
                break;
            case self::INBANK_CALC_SMALL:
                $vars['body'] = [
                    'marketing_message' => 'presta_indivy_product_page_period',
                ];
                break;
        }

        return $vars;
    }

    public function getPaymentTemplate($product_price, $calculations = [], &$placeholders = null)
    {
        $path = INBANK_MODULE_DIR.'cache/'.$this->object->id.'_'.(int)$this->object->debug_mode.'_'
            .self::INBANK_PAYMENT.'_'.Context::getContext()->language->id.'.html';
        $placeholders = [
            'checkbox_image' => INBANK_MODULE_URL.'views/img/checkbox-inbank.png',
            'default_period' => $this->object->period,
            'additional_details' => $this->object->custom_additional_details
        ];

        if (file_exists($path)) {
            return self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
        } else {
            $template = $this->object->down_payment ? self::INBANK_DOWNPAYMENT : self::INBANK_PAYMENT;
            $this->generateTemplate($template);
            if (file_exists($path)) {
                return self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
            }
        }

        return false;
    }

    public function getCalcSmallTemplate($product_price)
    {
        $path = INBANK_MODULE_DIR . 'cache/' . $this->object->id . '_' . (int)$this->object->debug_mode . '_'
            . self::INBANK_CALC_SMALL . '_' . Context::getContext()->language->id . '.html';

        /*
        $api_response = $this->api->getCreditCalculation([
            'amount' => $product_price
        ]);
        if ($api_response == false || isset($api_response->message) || isset($api_response->error)) {
            return false;
        }
        */

        $placeholders = [
            'marketing_message' => $this->object->title
        ];

        if ($this->object->use_logo) {
            $logo = $this->getLogo();
            $placeholders['image_link'] = '<img class="inbank_logo_product" src="' . $logo['url'] . '" style="' . $logo['style'] . '"/>';
        } else {
            $placeholders['image_link'] = '';
        }

        $template = '';
        if (file_exists($path)) {
            $template = self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
        } else {
            $this->generateTemplate(self::INBANK_CALC_SMALL);
            if (file_exists($path)) {
                $template = self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
            }
        }

        if (empty($template)) {
            return false;
        }

        return self::replacePlaceholders($template, ['payment_date' => date('25.m.Y', strtotime('+1 month'))]);
    }

    public function displayPaymentOption16($product_price, $currency_code)
    {
        $placeholders = [
            'payment_date' => date('25.m.Y', strtotime('+1 month'))
        ];
        $title = self::getDefaultTitle();
        if (!empty($this->object->custom_title)) {
            $title = self::replacePlaceholders($this->object->custom_title, $placeholders);
        } elseif (!empty($this->object->title)) {
            $title = self::replacePlaceholders($this->object->title, $placeholders);
        }

        $context = Context::getContext();
        $context->smarty->assign([
            'payment_title' => $title,
            'payment_class' => $this->object->class_name,
            'payment_image' => $this->object->use_logo ? $this->getLogoPath() : false,
            'payment_id' => $this->object->id,
            'use_logo' => $this->object->use_logo,
        ]);

        return $context->smarty->fetch(INBANK_MODULE_DIR.'views/templates/hook/payment_16.tpl');
    }

    public function displayPaymentOption17($product_price, $currency_code)
    {
        if (!$this->isAcceptedCurrency('EUR', $currency_code)) {
            return false;
        }
        $placeholders = [
            'payment_date' => date('25.m.Y', strtotime('+1 month'))
        ];
        $paymentOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $paymentOption->setModuleName($this->module->name);
        if ($this->object->use_logo) {
            $paymentOption->setLogo($this->getLogoPath(true));
        }
        /*
        $title = self::getDefaultTitle();
        if (!empty($this->object->custom_title)) {
            $title = self::replacePlaceholders($this->object->custom_title, $placeholders);
        } else if (!empty($this->object->title)) {
            $title = self::replacePlaceholders($this->object->title, $placeholders);
        }*/
        $title = self::replacePlaceholders($this->object->title, $placeholders);
        $paymentOption->setCallToActionText($title);
        $paymentOption->setAction(Context::getContext()->link->getModuleLink(
            $this->module->name,
            'validation',
            [
                'inbank_payment' => $this->object->id,
            ],
            true
        ));
        $paymentOption->setAdditionalInformation($this->getPaymentTemplate($product_price));
        return $paymentOption;
    }

    public function productApiForm()
    {
        $form = parent::productApiForm();
        foreach ($form['form']['input'] as $key => $input) {
            if ($input['name'] == 'custom_title') {
                $form['form']['input'][$key]['name'] = 'title';
                $form['form']['input'][$key]['disabled'] = true;
                $form['form']['input'][$key]['readonly'] = true;
            }
            if ($input['name'] == 'calc_small') {
                $form['form']['input'][$key]['type'] = 'switch';
            } elseif ($input['name'] == 'calc_small_position') {
                $form['form']['input'][$key]['type'] = 'select';
            }
        }

        return $form;
    }
}
