<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankIndivySliceLV extends InbankBalticsProduct
{

    private $class_name = 'InbankIndivySliceLV';
    public $production_api = 'https://api.inbank.lv/partner/v2/';
    public $debug_api = 'https://demo-api.inbank.lv/partner/v2/';
    public $has_large_calc = false;
    public $product_code = '';
    public $default_logo_name = 'inbank-logo.svg';
    public $subtype = 'slice';


    public static function getDefaultTitle()
    {
        return 'Inbank Split into parts';
    }

    public static function getAvailableCountries()
    {
        return 'lv';
    }

    public static function getSubtype()
    {
        return 'slice';
    }

    public static function getPhraseTemplateVars($template, $additional_info = [])
    {
        $default_period = 0;
        $vars = array();
        if (isset($additional_info['default_period'])) {
            $default_period = $additional_info['default_period'];
        }
        if ($default_period == 3) {
            $vars['title'] = 'presta_plugin_product_slice_marketingMessage_3month';
        } else {
            $vars['title'] = 'presta_plugin_product_slice_marketingMessage';
        }
        $vars['additional_details'] = 'presta_plugin_product_slice_consent';
        $vars['granted'] = 'presta_plugin_product_granted_text';
        switch ($template) {
            case self::INBANK_PAYMENT:
                $vars['body'] = [
                    'down_payment' => 'presta_plugin_downpayment_message',
                    'selling_point_1' => 'presta_slice_selling_point_one',
                    'selling_point_2' => 'presta_slice_selling_point_two',
                    'selling_point_3' => 'presta_slice_selling_point_three',
                ];
                break;
            case self::INBANK_CALC_SMALL:
                if ($default_period == 3) {
                    $vars['body'] = [
                        'marketing_message' => 'presta_plugin_product_slice_marketingMessage_3month',
                    ];
                } else {
                    $vars['body'] = [
                        'marketing_message' => 'presta_plugin_product_slice_marketingMessage',
                    ];
                }
                break;
        }

        return $vars;
    }

    public function getPaymentTemplate($product_price, $calculations = [], &$placeholders = null)
    {
        $path = INBANK_MODULE_DIR.'cache/'.$this->object->id.'_'.(int)$this->object->debug_mode.'_'
            .self::INBANK_PAYMENT.'_'.Context::getContext()->language->id.'.html';
        $placeholders = [
            'checkbox_image' => INBANK_MODULE_URL.'views/img/checkbox-inbank.png',
            'default_period' => $this->object->default_period,
            'additional_details' => $this->object->custom_additional_details
        ];

        if (file_exists($path)) {
            return self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
        } else {
            $template = $this->object->down_payment ? self::INBANK_DOWNPAYMENT : self::INBANK_PAYMENT;
            $this->generateTemplate($template);
            if (file_exists($path)) {
                return self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
            }
        }

        return false;
    }

    //styling payment option for prestashop 1.6
    public function displayPaymentOption16($product_price, $currency_code)
    {
        if ($this->object->default_period == 3) {
            // because slice 3 uses different title template, we have to get more info from api
            $calculations = $this->api->getCreditCalculation([
                'amount' => $product_price,
                'response_level' => 'payment_schedule',
                'period' => $this->object->default_period
            ]);
        } else {
            $calculations = $this->api->getCreditCalculation([
                'amount' => $product_price,
                'period' => $this->object->default_period
            ]);
        }
        // we validate the api info
        if (!isset($calculations->product_code) || $calculations->product_code !== $this->getProductCode()) {
            return false;
        }
        $currency_symbol = $this->getCurrencySymbolByCode($calculations->currency);
        $title = self::getDefaultTitle();
        if (!empty($this->object->custom_title)) {
            $title = $this->object->custom_title;
        } elseif (!empty($this->object->title)) {
            $title = $this->object->title;
        }

        if ($this->object->default_period == 3) {
            $payment_title = self::replacePlaceholders(
                $title,
                [
                    'payment_one' => number_format($calculations->payment_schedule[1]->total_payment, 2) . ' ' . $currency_symbol,
                    'payment_two' => number_format($calculations->payment_schedule[2]->total_payment, 2) . ' ' . $currency_symbol,
                    'payment_three' => number_format($calculations->payment_schedule[3]->total_payment, 2) . ' ' . $currency_symbol,
                ]
            );
        } else {
            $title = self::replacePlaceholders(
                $this->object->title,
                ['default_period' => $this->object->default_period]
            );
        }

        $context = Context::getContext();
        $context->smarty->assign([
            'payment_title' => $title,
            'payment_class' => $this->object->class_name,
            'payment_image' => $this->object->use_logo ? $this->getLogoPath() : false,
            'payment_id' => $this->object->id,
            'use_logo' => $this->object->use_logo,
        ]);

        return $context->smarty->fetch(INBANK_MODULE_DIR.'views/templates/hook/payment_16.tpl');
    }

    public function displayPaymentOption17($product_price, $currency_code)
    {
        if (!$this->isAcceptedCurrency('EUR', $currency_code)) {
            return false;
        }
        $calculations = $this->api->getCreditCalculation([
            'amount' => $product_price,
            'currency' => $currency_code,
            'response_level' => 'payment_schedule',
            'period' => $this->object->default_period
        ]);
        if (!isset($calculations->product_code) && !isset($calculations->total_cost)) {
            return false;
        }
        $currency_symbol = $this->getCurrencySymbolByCode($calculations->currency);
        $placeholders = [];
        if ($this->object->default_period == 3) {
            if (isset($calculations->payment_schedule[1]->total_payment)) {
                $placeholders = [
                    'image_link' => $this->getLogoPath(),
                    'payment_one' => number_format($calculations->payment_schedule[1]->total_payment, 2).' '.$currency_symbol,
                    'payment_two' => number_format($calculations->payment_schedule[2]->total_payment, 2).' '.$currency_symbol,
                    'payment_three' => number_format($calculations->payment_schedule[3]->total_payment, 2).' '.$currency_symbol,
                    'down_payment' => number_format(
                        (float)($calculations->down_payment_minimum_amount ?? 0),
                        2
                        ).' '.$currency_symbol
                ];
            }
        } else {
            $placeholders = [
                'image_link' => $this->getLogoPath(),
                'default_period' => $this->object->default_period,
                'down_payment' => number_format(
                    (float)($calculations->down_payment_minimum_amount ?? 0),
                    2
                    ).' '.$currency_symbol
            ];
        }

        $paymentOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $paymentOption->setModuleName($this->module->name);
        if ($this->object->use_logo) {
            $paymentOption->setLogo($this->getLogoPath(true));
        }
        /*
        $title = self::getDefaultTitle();
        if (!empty($this->object->custom_title)) {
            $title = self::replacePlaceholders($this->object->custom_title, $placeholders);
        } else if (!empty($this->object->title)) {
            $title = self::replacePlaceholders($this->object->title, $placeholders);
        }*/
        $title = self::replacePlaceholders($this->object->title, $placeholders);
        $paymentOption->setCallToActionText($title);
        $paymentOption->setAction(Context::getContext()->link->getModuleLink(
            $this->module->name,
            'validation',
            [
                'inbank_payment' => $this->object->id,
            ],
            true
        ));
        $paymentOption->setAdditionalInformation($this->getPaymentTemplate($product_price));

        return $paymentOption;
    }

    public function getCalcSmallTemplate($product_price)
    {
        $path = INBANK_MODULE_DIR.'cache/'.$this->object->id.'_'.(int)$this->object->debug_mode.'_'
            .self::INBANK_CALC_SMALL.'_'.Context::getContext()->language->id.'.html';
        $api_response = $this->api->getCreditCalculation([
            'amount' => $product_price,
            'response_level' => 'payment_schedule',
            'period' => $this->object->default_period
        ]);
        if ($api_response == false || isset($api_response->message)) {
            return false;
        }
        if ($this->object->default_period == 3) {
            if (!isset($api_response->payment_schedule[1]->total_payment)) {
                return false;
            }
            $placeholders = [
                'payment_one' => $api_response->payment_schedule[1]->total_payment.' &euro;',
                'payment_two' => $api_response->payment_schedule[2]->total_payment.' &euro;',
                'payment_three' => $api_response->payment_schedule[3]->total_payment.' &euro;',
            ];
        } else {
            $placeholders = [
                'default_period' => $this->object->default_period,
            ];
        }
        if ($this->object->use_logo) {
            $logo = $this->getLogo();
            $placeholders['image_link'] = '<img class="inbank_logo_product" src="'.$logo['url'].'" style="'.$logo['style'].'"/>';
        } else {
            $placeholders['image_link'] = '';
        }
        if (file_exists($path)) {
            return self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
        } else {
            $this->generateTemplate(self::INBANK_CALC_SMALL);
            if (file_exists($path)) {
                return self::replacePlaceholders(Tools::file_get_contents($path), $placeholders);
            }
        }

        return false;
    }

    public function productApiForm()
    {
        $form = parent::productApiForm();
        foreach ($form['form']['input'] as $key => $input) {
            if ($input['name'] == 'custom_title') {
                $form['form']['input'][$key]['name'] = 'title';
                $form['form']['input'][$key]['disabled'] = true;
                $form['form']['input'][$key]['readonly'] = true;
            }
        }

        return $form;
    }
}
